import inspect
import json
import traceback
import unittest
from datetime import date
from datetime import datetime, timedelta

#from datetime import datetime, timedelta
#from dateutil.parser import parse as parse_date
import datetime

from bravado.client import SwaggerClient
from bravado.requests_client import RequestsClient

class Exposures(object):

    def __init__(self, swagger_endpoint_url="https://exposures.renci.org/v1/swagger.json"):
        http_client = RequestsClient ()
        # TODO: fix ssl certificate.
#        http_client.session.verify = False
        self.client = SwaggerClient.from_url(
            swagger_endpoint_url,
            http_client=http_client)

    def inspect (self):
        for name, obj in inspect.getmembers (self.client):
            print ("-- name: {0} obj: {1}".format (name, obj))
            for n, o in inspect.getmembers (obj):
                if (n.endswith ('_get') or n.endswith ('_post')):
                    print ("-- INVOKE: method-> {0} obj: {1}".format (n, o))

    def serialize (self, exposures):
        result = []
        for e in exposures:
            obj = {
                "exposure_type" : e.exposure_type,
                "latitude"      : e.latitude,
                "longitude"     : e.longitude,
                "start_time"    : e.start_time.strftime ("%Y-%m-%d"),
                "end_time"      : e.end_time.strftime ("%Y-%m-%d"),
                "value"         : e.value,
                "units"         : e.units
            }
            result.append (obj)
        return json.dumps (result)
            
    def get_coordinates (self, exposure_type, latitude, longitude, radius):
        return self.client.default.controllers_default_controller_exposures_exposure_type_coordinates_get (
            exposure_type=exposure_type,
            latitude=latitude,
            longitude=longitude,
            radius=radius).result ()

    def get_scores (self, exposure_type, start_date, end_date, exposure_point):
        #print ("exposures .............> {}".format (start_date))
         return self.serialize (self.client.default.controllers_default_controller_exposures_exposure_type_scores_get (
            exposure_type=exposure_type,
            start_date=start_date,
            end_date=end_date,
            exposure_point=exposure_point).result ())

    def get_values (self, exposure_type, start_date, end_date, exposure_point):
        return self.serialize (self.client.default.controllers_default_controller_exposures_exposure_type_values_get (
            exposure_type=exposure_type,
            start_date=start_date,
            end_date=end_date,
            exposure_point=exposure_point).result ())
            
class TestExposures(unittest.TestCase):

    exposures = Exposures ()

    def test_get_coordinates (self):
        print ("Test get coordinates")
        coordinates = self.exposures.get_coordinates (
            exposure_type='pm25',
            latitude='35.9131996',
            longitude='-79.0558445',
            radius='10')
        for coordinate in coordinates:
            print ("  --coord: lat: {0} lon: {1}".format (coordinate.latitude, coordinate.longitude))

    def test_get_scores (self):
        print ("Test get scores")
        scores = self.exposures.get_scores (
            exposure_type='pm25',
            start_date=date(2010, 1, 1),
            end_date=date(2010, 1, 7),
            exposure_point='35.9131996,-79.0558445')
        for exposure in scores:
            print ("  --scores: start: {0} end: {1} type: {2} lat: {3} lon: {4} units: {5} value: {6}".format (
                exposure.start_time, exposure.end_time, exposure.exposure_type,
                exposure.latitude, exposure.longitude, exposure.units, exposure.value))

    def test_get_values (self):
        print ("Test get values")
        values = self.exposures.get_values (
            exposure_type='pm25',
            start_date=date(2010, 1, 1),
            end_date=date(2010, 1, 7),
            exposure_point='35.9131996,-79.0558445')
        for exposure in values:
            print ("  --values: start: {0} end: {1} type: {2} lat: {3} lon: {4} units: {5} value: {6}".format (
                exposure.start_time, exposure.end_time, exposure.exposure_type,
                exposure.latitude, exposure.longitude, exposure.units, exposure.value))

if __name__ == '__main__':
    unittest.main()



from greent.service import Service
from greent.service import ServiceContext
from ontobio.ontol_factory import OntologyFactory
from greent.graph_components import KNode, KEdge
from greent.util import LoggingUtil
from greent import node_types
from cachier import cachier
import datetime


logger = LoggingUtil.init_logging (__file__)

#TODO: LOOKUP all the terms that map to this... or use an ancestor call that doesn't require such stuff (i.e. that handles this)
GENETIC_DISEASE=('DOID:630','http://purl.obolibrary.org/obo/EFO_0000508','MONDO:0003847','http://purl.obolibrary.org/obo/MONDO_0003847')
#GENETIC_DISEASE='EFO:0000508'
MONOGENIC_DISEASE='DOID:0050177'

class Mondo(Service):
    
    """ A pragmatic class to query the mondo ontology. Until better sources emerge, we roll our own. """ 
    def __init__(self, context ):
        super(Mondo, self).__init__("mondo", context)
        ofactory = OntologyFactory()
        try:
            #sometimes the ontology world is down :(
            self.ont = ofactory.create('mondo')
        except:
            logger.warn('Problem reaching sparql endpoint, falling back to obo')
            try:
                self.ont = ofactory.create('obo:mondo')
            except:
                logger.warn('Problem reaching obo, falling back to local owl file')
                self.ont = ofactory.create('onto_cache/mondo.owl')
        #self.ont = ofactory.create('./mondo.owl')
        #This seems to be required to make the ontology actually load:
        _ = self.ont.get_level(0)
        
    def get_doid(self,identifier):
        """We have an identifier, and we are going to use MONDO to try to convert it to a DOID"""
        upper_id = identifier.upper()
        obj_ids = self.get_mondo_id(upper_id)
        #Are any of the ids we get back a DOID?
        doids = []
        for obj_id in obj_ids:
            if obj_id.startswith('DOID:'):
                doids.append(obj_id)
        if len(doids) > 0:
            return doids
        #Didn't get anything, so get the xrefs and find DOIDS
        for obj_id in obj_ids:
            xref_ids = self.ont.xrefs(obj_id)
            for xref_id in xref_ids:
                if xref_id.startswith('DOID:'):
                    doids.append( xref_id )
        return doids


    def mondo_get_doid(self,mondo_identifier):
        return self.mondo_get_synonym(mondo_identifier,'DOID')

    def mondo_get_umls(self,mondo_identifier):
        return self.mondo_get_synonym(mondo_identifier,'UMLS')

    def mondo_get_efo(self,mondo_identifier):
        return self.mondo_get_synonym(mondo_identifier,'EFO')

    def mondo_get_synonym(self,mondo_identifier,curie_prefix):
        xref_ids = self.ont.xrefs(mondo_identifier)
        doids = []
        for xref_id in xref_ids:
            if xref_id.startswith(curie_prefix):
                doids.append( xref_id )
        return doids

    def get_label(self,identifier):
        """Return the label for an identifier"""
        obj_ids = self.get_mondo_id(identifier)
        return self.ont.label(obj_ids[0])

    def get_mondo_id(self,obj_id):
        """Given an id, find the main key(s) that mondo uses for the id"""
        if self.ont.has_node(obj_id):
            obj_ids = [obj_id]
        else:
            obj_ids = self.ont.xrefs(obj_id, bidirectional=True)
        return obj_ids

    def has_ancestor(self,obj, terms):
        """Given an object and a term in MONDO, determine whether the term is an ancestor of the object.
        
        The object is a KNode representing a disease.
        Some complexity arises because the identifier for the node may not be the id of the concept in mondo.
        the XRefs in mondo are checked for the object if it is not intially found, but this may return more 
        than one entity if multiple mondo entries map to the same.  

        Returns: boolean representing whether any mondo objects derived from the subject have the term as an
                         ancestor.
                 The list of Mondo identifiers for the object, which have the term as an ancestor"""
        #TODO: The return signature is funky, fix it.
        obj_id = obj.identifier
        obj_ids = self.get_mondo_id(obj_id)
        return_objects=[]
        for obj_id in obj_ids:
            ancestors = self.ont.ancestors(obj_id)
            for term in terms:
                if term in ancestors:
                    return_objects.append( obj_id )
        return len(return_objects) > 0, return_objects

    def is_genetic_disease(self,obj):
        """Checks mondo to find whether the subject has DOID:630 as an ancestor"""
        return self.has_ancestor(obj, GENETIC_DISEASE)

    def is_monogenic_disease(self,obj):
        """Checks mondo to find whether the subject has DOID:0050177 as an ancestor"""
        return self.has_ancestor(obj, MONOGENIC_DISEASE)

    #@cachier(stale_after=datetime.timedelta(days=20))
    def doid_get_genetic_condition (self, disease):
        """Given a gene specified as an HGNC curie, return associated genetic conditions.
        A genetic condition is specified as a disease that descends from a ndoe for genetic disease in MONDO."""
        relations = []
        is_genetic_condition, new_object_ids = self.is_genetic_disease (disease)
        orphanet_prefix = "http://www.orpha.net/ORDO/Orphanet_"
        if is_genetic_condition:
            for new_object_id in new_object_ids:
                if new_object_id.startswith (orphanet_prefix):
                    new_object_id = new_object_id.replace (orphanet_prefix, 'ORPHANET.GENETIC_CONDITION:')
                elif new_object_id.startswith ('DOID:'):
                    new_object_id = new_object_id.replace ('DOID:', 'DOID.GENETIC_CONDITION:')
                relations.append ( (self.get_edge ({}, 'is_genetic_condition'), KNode (new_object_id, node_types.GENETIC_CONDITION) ))
        return relations

    def doid_get_orphanet_genetic_condition (self, disease):
        results = self.doid_get_genetic_condition (disease)
        return [ r for r in results if r[1].identifier.startswith ('ORPHANET.GENETIC_CONDITION') ]
    
    def doid_get_doid_genetic_condition (self, disease):
        results = self.doid_get_genetic_condition (disease)
        return [ r for r in results if r[1].identifier.startswith ('DOID.GENETIC_CONDITION') ]

    def substring_search(self,name):
        ciname = '(?i){}'.format(name)
        results = self.ont.search(ciname,synonyms=True,is_regex=True)

    def case_insensitive_search(self,name):
        ciname = '(?i)^{}$'.format(name)
        return self.ont.search(ciname,synonyms=True,is_regex=True)

    def search(self, name):
        #Exact match 
        results = self.case_insensitive_search(name)
        if len(results) == 0:
            if ',' in name:
                parts =name.split(',')
                parts.reverse()
                ps = [p.strip() for p in parts]
                newname = ' '.join(ps)
                results = self.case_insensitive_search(newname)
        return results


def test_both():
    q1in='q1-disease-list.txt'
    q1out='q1_disease_mondo.txt'
    q1field = 0
    test_one(q1in, q1out, q1field)
    q2in='q2-drugandcondition-list.txt'
    q2out='q2_disease_mondo.txt'
    q2field = 1
    test_one(q2in, q2out, q2field)
    
def test_one(infname,outfname,fieldnum):
    m = Mondo (ServiceContext.create_context ())
    n_good = 0
    n_bad = 0
    diseases = set()
    with open(infname,'r') as inf, open(outfname,'w') as outf:
        h = inf.readline()
        for line in inf:
            if line.startswith('#'):
                continue
            x = line.strip().split('\t')[fieldnum]
            if x in diseases:
                continue
            diseases.add(x)
            result = m.search (x)
            if len(result) == 0:
                mondos = ''
                names = ''
                doids = ''
                umlss = ''
                efos  = ''
                n_bad += 1
            else:
                n_good += 1
                mondos = ';'.join( result )
                names = ';'.join([ m.get_label( r ) for r in result ])
                doids = ';'.join(sum([ m.mondo_get_doid( r ) for r in result ], [] ))
                umlss = ';'.join(sum([ m.mondo_get_umls( r ) for r in result ], [] ))
                efos = ';'.join(sum([ m.mondo_get_efo( r ) for r in result ], [] ))
            outf.write('{}\t{}\t{}\t{}\t{}\n'.format(x, mondos, doids, umlss, efos ))
            print( 'Good: {}   Bad: {}'.format(n_good, n_bad) )


def test():
    m = Mondo (ServiceContext.create_context ())
    huntington = KNode('OMIM:143100',node_types.DISEASE)
    print(m.is_genetic_disease(huntington))
    print('------')
#    alc_sens = KNode('OMIM:610251',node_types.DISEASE)
#    print(m.is_genetic_disease(alc_sens))
#    print('------')
#    tests = [ "DOID:8545", "OMIM:218550", "OMIM:234000", "DOID:0060334", "DOID:0050524", "DOID:0060599", "DOID:12858" ]
#    for t in tests:
#        print (m.doid_get_orphanet_genetic_condition (KNode (t, node_types.DISEASE)))

if __name__ == '__main__':
    test_both()



import logging
import requests
import pprint
import urllib
from greent.service import Service
from greent.service import ServiceContext
from greent.util import Text
from greent.graph_components import KNode,KEdge
from greent import node_types

class QuickGo(Service):

    def __init__(self, context):
        super(QuickGo, self).__init__("quickgo", context)

    def go_term_xontology_relationships(self, node):
        #Many of the nodes coming in will be something like GO.BIOLOGICAL_PROCESS:0042626 and
        # need to be downgraded to just GO
        url = "{0}/QuickGO/services/ontology/go/terms/GO:{1}/xontologyrelations".format (self.url, Text.un_curie(node.identifier))
        response = requests.get(url).json ()
        results = []
        for r in response['results']:
            if 'xRelations' in r:
                for xrel in r['xRelations']:
                    if xrel['id'].startswith('CL:'):
                        results.append( ( self.get_edge (r, predicate=xrel['relation']), KNode (xrel['id'], node_types.CELL, label = xrel['term']) ))
        return results

    def go_term_annotation_extensions(self,node):
        """This is playing a little fast and loose with the annotations.  Annotations relate a gene to a go term,
        and they can have an extension like occurs_in(celltype). Technically, that occurs_in only relates to that
        particular gene/go combination.  But it's the only way to traverse from neurotransmitter release to neurons 
        that is currently available"""
        url = '{0}/QuickGO/services/annotation/search?includeFields=goName&goId=GO:{1}&taxonId=9606&extension=occurs_in(CL)'.format( self.url, Text.un_curie(node.identifier)) 
        response = requests.get(url).json()
        results = []
        cell_ids = set()
        for r in response['results']:
            for e in r['extensions']:
                for c in e['connectedXrefs']:
                    if c['db'] == 'CL':
                        if c['id'] not in cell_ids:
                            results.append( (self.get_edge( r, predicate=c['qualifier']), 
                                KNode( 'CL:{}'.format(c['id']), node_types.CELL ) ) )
                            cell_ids.add(c['id'])
        return results

def test ():
    q = QuickGo (ServiceContext.create_context ())
    r = q.go_term_xontology_relationships (KNode("GO:0002551", node_types.PROCESS))
    pprint.pprint (r)
    r = q.go_term_xontology_relationships (KNode("GO.BIOLOGICAL_PROCESS:0042626", node_types.PROCESS))
    pprint.pprint (r)
    r = q.go_term_annotation_extensions (KNode("GO.BIOLOGICAL_PROCESS:0007269", node_types.PROCESS))
    pprint.pprint (r)

if __name__ == '__main__':
    test ()



import logging
import inspect
import json
import requests
import traceback
import unittest
import datetime
import os
import yaml
from collections import namedtuple
from bravado.client import SwaggerClient
from bravado.requests_client import RequestsClient

class LoggingUtil(object):
    """ Logging utility controlling format and setting initial logging level """
    @staticmethod
    def init_logging (name, level=logging.INFO, format='short'):
        FORMAT = {
            "short" : '%(funcName)s: %(message)s',
            "long"  : '%(asctime)-15s %(filename)s %(funcName)s %(levelname)s: %(message)s'
        }[format]
        logging.basicConfig (format=FORMAT, level=logging.WARNING)
        logger = logging.getLogger (name)
        logger.setLevel(level)
        return logger
    
class Munge(object):
    @staticmethod
    def gene (gene):
        return gene.split ("/")[-1:][0] if gene.startswith ("http://") else gene
    
class Text:
    """ Utilities for processing text. """

    @staticmethod
    def get_curie (text):
        return text.upper ().split (':')[0] if ':' in text else None
        
    @staticmethod
    def un_curie (text):
        return text.split (':')[1] if ':' in text else text
        
    @staticmethod
    def short (obj, limit=80):
        text = str(obj) if obj else None
        return (text[:min(len(text),limit)] + ('...' if len(text)>limit else '')) if text else None

    @staticmethod
    def path_last (text):
        return text.split ('/')[-1:][0] if '/' in text else text

    @staticmethod
    def obo_to_curie (text):
        return ':'.join( text.split('/')[-1].split('_') )

class Resource:
    @staticmethod
    def get_resource_path(resource_name):
        """ Given a string resolve it to a module relative file path unless it is already an absolute path. """
        resource_path = resource_name
        if not resource_path.startswith (os.sep):
            resource_path = os.path.join (os.path.dirname (__file__), resource_path)
        return resource_path
    @staticmethod
    def load_json (path):
        result = None
        with open (path, 'r') as stream:
            result = json.loads (stream.read ())
        return result

    @staticmethod
    def load_yaml (path):
        result = None
        with open (path, 'r') as stream:
            result = yaml.load (stream.read ())
        return result
    
    def get_resource_obj (resource_name, format='json'):
        result = None
        path = Resource.get_resource_path (resource_name)
        if os.path.exists (path):
            m = {
                'json' : Resource.load_json,
                'yaml' : Resource.load_yaml
            }
            if format in m:
                result = m[format](path)
        return result
    
class DataStructure:
    @staticmethod
    def to_named_tuple (type_name, d):
        return namedtuple(type_name, d.keys())(**d)



import requests
import json
import urllib
from greent.service import Service
from greent.service import ServiceContext
from greent.mondo import Mondo
from greent.go import GO
from greent.util import Text
from greent.graph_components import KNode,KEdge
from greent import node_types
import logging

class Biolink(Service):
    """ Preliminary interface to Biolink. Will move to automated Translator Registry invocation over time. """
    def __init__(self, context):
        super(Biolink, self).__init__("biolink", context)
        #TODO, can we just use the Mondo that's inthe core already?
        self.checker = Mondo(ServiceContext.create_context ())
        self.go = GO(ServiceContext.create_context ())
    def process_associations(self, r, predicate, target_node_type):
        edge_nodes = [ ]
        for association in r['associations']:
            pubs = []
            if 'publications' in association and association['publications'] is not None:
                for pub in association['publications']:
                    #Sometimes, we get back something like "uniprotkb" instead of a PMID.  We don't want it.
                    pubid_prefix = pub['id'][:4].upper()
                    if pubid_prefix == 'PMID':
                        pubs.append( pub['id'] )
            obj = KNode(association['object']['id'], target_node_type, association['object']['label'] )
            rel = { 'typeid': association['relation']['id'], 'label':association['relation']['label'] }
            props = { 'publications': pubs, 'relation':rel }
            edge = KEdge( 'biolink', predicate , props )
            edge_nodes.append( (edge , obj ) )
        return edge_nodes
    def gene_get_disease(self, gene_node):
        """Given a gene specified as an HGNC curie, return associated diseases. """
        ehgnc = urllib.parse.quote_plus(gene_node.identifier)
        logging.getLogger('application').debug('          biolink: %s/bioentity/gene/%s/diseases' % (self.url, ehgnc))
        r = requests.get('%s/bioentity/gene/%s/diseases' % (self.url, ehgnc)).json()
        return self.process_associations( r, 'gene_get_disease', node_types.DISEASE )
    def disease_get_phenotype(self,disease):
        url = "{0}/bioentity/disease/{1}/phenotypes/".format (self.url, disease.identifier )
        response = requests.get (url).json ()
        return self.process_associations( response, 'disease_get_phenotype', node_types.PHENOTYPE )
    def gene_get_go(self,gene):
        #this function is very finicky.  gene must be in uniprotkb, and the curie prefix must be correctly capitalized
        url = "{0}/bioentity/gene/UniProtKB:{1}/function/".format (self.url, Text.un_curie(gene.identifier) )
        response = requests.get (url).json ()
        #return [ (a['object']['id'] , a['object']['label']) for a in response['associations'] ]
        return self.process_associations(response, 'gene_get_go', node_types.PROCESS)
    def gene_get_function (self, gene):
        edges_nodes = self.gene_get_go( gene )
        process_results = list( filter( lambda x: self.go.is_molecular_function(x[1].identifier), edges_nodes ) )
        for edge, node in process_results:
            edge.predicate='gene_get_molecular_function'
            node.identifier.replace('GO:', 'GO.MOLECULAR_FUNCTION:')
            node.node_type = node_types.FUNCTION
        return process_results
    def gene_get_process ( self, gene):
        edges_nodes = self.gene_get_go( gene )
        process_results = list( filter( lambda x: self.go.is_biological_process(x[1].identifier), edges_nodes ) )
        for edge, node in process_results:
            edge.predicate='gene_get_biological_process'
            node.identifier.replace('GO:', 'GO.BIOLOGICAL_PROCESS:')
            node.node_type = node_types.PROCESS
        return process_results

    def gene_get_genetic_condition(self, gene):
        """Given a gene specified as an HGNC curie, return associated genetic conditions.
        A genetic condition is specified as a disease that descends from a ndoe for genetic disease in MONDO."""
        disease_relations = self.gene_get_disease(gene)
        #checker = Mondo(ServiceContext.create_context ())
        relations = []
        for relation, obj in disease_relations:
            is_genetic_condition, new_object_ids = self.checker.is_genetic_disease(obj)
            if is_genetic_condition:
                obj.properties['mondo_identifiers'] = new_object_ids
                obj.node_type = node_types.GENETIC_CONDITION
                relations.append( (relation,obj) )
        #print (" biolink relations %s" % relations)
        return relations

def test():
    """What do we get back for HBB"""
    relations = gene_get_disease(('HGNC:4827',))
    checker = Mondo(ServiceContext.create_context ())
    for p, a in relations:
        igc, nid = checker.is_genetic_disease(a)
        print(a['id'], igc, nid)

def test_output():
    ehgnc = urllib.parse.quote_plus("HGNC:6136")
    r = requests.get('https://api.monarchinitiative.org/api/bioentity/gene/%s/diseases' % ehgnc).json()
    import json
    with open('testbiolink.json','w') as outf:
        json.dump(r, outf, indent=4)

def test_go():
    KIT_protein = KNode('UniProtKB:P10721',node_types.GENE)
    b = Biolink (ServiceContext.create_context ())
    results=b.gene_get_process(KIT_protein)
    for ke, kn in results:
        print(ke, kn)

def test_phenotypes():
    asthma = KNode('DOID:2841', node_types.DISEASE)
    b = Biolink (ServiceContext.create_context ())
    results=b.disease_get_phenotype(asthma)
    for ke, kn in results:
        print(ke, kn)

def test_gc():
    b = Biolink (ServiceContext.create_context ())
    gene = KNode('HGNC:4851',node_type=node_types.GENE)
    results = b.gene_get_disease(gene)
    for e,k in results:
        print(k)
    results = b.gene_get_genetic_condition(gene)
    print(results)

if __name__ == '__main__':
    test_gc()
    #test_output()
    #b = Biolink (ServiceContext.create_context ())
#    print (b.get_gene_function (KNode('UniProtKB:P10721', 'G')))
    #print (b.gene_get_genetic_condition (KNode ('DOID:2841', node_types.DISEASE)))



import requests
import json
from greent.service import Service
from greent.service import ServiceContext
from greent.util import LoggingUtil
from greent.util import Text
from greent.graph_components import KNode,KEdge
from greent import node_types

logger = LoggingUtil.init_logging (__name__)

class TranslatorKnowledgeBeaconAggregator(Service):
    """ Use Translator Knowledge Beacons via the aggregator. """

    def __init__(self, context):
        super(TranslatorKnowledgeBeaconAggregator, self).__init__("tkba", context)

    def request_concept (self, concept, stype=None):
        #Without quotes around the keyword, this function treats space as a delimiter...
        keyword = Text.un_curie (concept.identifier)
        keyword = '"{0}"'.format (keyword) if ' ' in keyword else keyword
        if stype is None:
            url = '{0}/concepts?keywords={1}'.format (self.url, keyword)
        elif stype == node_types.DISEASE:
            url = '{0}/concepts?keywords={1}&semanticGroups=DISO'.format (self.url, keyword)
        else:
            url = '{0}/concepts?keywords={1}'.format (self.url, keyword)
        return requests.get (url).json ()

    def name_to_doid (self, name):
        result = []
        response = self.request_concept (name, node_types.DISEASE)
        seen = {}
        for r in response:
            got_doid = False
            for a in r['aliases']:
                if a.startswith ("DOID:"):
                    got_doid = True
                    if not a in seen:
                        logger.debug ("      -- appending a {}".format (a))
                        result.append ( ( self.get_edge (r, predicate='name_to_doid'), KNode(a, node_types.DISEASE ) ) )
                        seen[a] = a
        logger.info("Returning {} doids".format(len(result)))
        return result

    def name_to_anything (self, name):
        result = []
        response = self.request_concept (name, node_types.DISEASE)
        for r in response:
            return ';'.join(r['aliases'])

    def name_to_efo (self, name):
        result = []
        response = self.request_concept (name)
        seen = {}
        for r in response:
            for a in r['aliases']:
                if a.startswith ("EFO:"):
                    if not a in seen:
                        logger.debug ("      -- appending a {}".format (a))
                        result.append ( ( self.get_edge (r, predicate='name_to_efo'), KNode(a, node_types.DISEASE ) ) )
                        seen[a] = a
        return result

    def name_to_drugbank (self, name):
        response = self.request_concept (name)
        result = []
        seen = {}
        for r in response:
            for a in r['aliases']:
                if a.startswith ("DRUGBANK:"):
                    if not a in seen:
                        result.append ( ( self.get_edge (r, predicate='name_to_drugbank'), KNode(a, node_types.DRUG) ) )
                        seen[a] = a
        return list(set(result))
        
    def name_to_mesh (self, name):
        result = []
        response = self.request_concept (name)
        seen = {}
        for r in response:
            for a in r['aliases']:
                if a.startswith ("MESH:"):
                    if not a in seen:
                        #TODO: not sure what node type should be here...
                        result.append ( ( self.get_edge (r, predicate='name_to_mesh'), KNode(a, name.node_type) ) )
                        seen[a] = a
        return list(set(result))
    
    def name_to_mesh_disease (self, name):
        response = self.request_concept (name)
        is_a_disease = False
        for r in response:
            for a in r['aliases']:
                if a.startswith ("DOID:"):
                    is_a_disease = True
                    break
        result = []
        if is_a_disease:
            seen = {}
            for r in response:
                for a in r['aliases']:
                    if a.startswith ("MESH:"):
                        if not a in seen:
                            a = a.replace ("MESH:", "MESH.DISEASE:")
                            result.append ( ( self.get_edge (r, predicate='name_to_mesh_disease'), \
                                KNode(a, node_types.DISEASE) ) )
                            seen[a] = a
        return list(set(result))

def basic_test():
    t = TranslatorKnowledgeBeaconAggregator (ServiceContext.create_context ())
    #print (t.name_to_mesh_disease (KNode("NAME.DISEASE:asthma", node_types.NAME_DISEASE)))
    #print (t.name_to_doid (KNode("NAME.DISEASE:asthma", node_types.DISEASE)))
    print ('1.')
    print (t.name_to_doid (KNode("NAME.DISEASE:Osteoporosis", node_types.DISEASE)))
    print ('2.')
    print (t.name_to_doid (KNode("NAME.DISEASE:HIV infection", node_types.DISEASE)))
    print ('3.')
    print (t.name_to_efo (KNode("NAME.DISEASE:HIV infection", node_types.DISEASE)))

def test_q2_diseases():
    t = TranslatorKnowledgeBeaconAggregator (ServiceContext.create_context ())
    n_good = 0
    n_bad = 0
    diseases = set()
#    with open('q2-drugandcondition-list.txt','r') as inf, open('q2_disease_report.txt','w') as outf:
#        h = inf.readline()
#        outf.write('OriginalName\tDOIDs\tOthers\n')
#        for line in inf:
#            x = line.strip().split('\t')[1]
    with open('q1-disease-list.txt','r') as inf, open('q1_disease_report.txt','w') as outf:
        h = inf.readline()
        outf.write('OriginalName\tDOIDs\tOthers\n')
        for line in inf:
            x = line.strip().split('\t')[0]
            if x in diseases:
                continue
            diseases.add(x)
            node = KNode("NAME.DISEASE:{}".format(x), node_types.DISEASE)
            result = t.name_to_doid (node)
            if len(result) == 0:
                doids = ''
                aliases = t.name_to_anything(node)
                n_bad += 1
            else:
                n_good += 1
                doids = ';'.join( [r[1].identifier for r in result] )
                aliases = ''
            outf.write('{}\t{}\t{}\n'.format(x, doids, aliases))
            print( 'Good: {}   Bad: {}'.format(n_good, n_bad) )

if __name__ == "__main__":
    test_q2_diseases()



from greent.service import Service
from greent.service import ServiceContext
from ontobio.ontol_factory import OntologyFactory
from greent.graph_components import KNode, KEdge
from greent.util import LoggingUtil
from greent import node_types
from cachier import cachier
import datetime


logger = LoggingUtil.init_logging (__file__)

#TODO: move some some of the common stuff between here and Mondo into an ontology class/module.
class HPO(Service):
    
    """ A pragmatic class to query the mondo ontology. Until better sources emerge, we roll our own. """ 
    def __init__(self, context ):
        super(HPO, self).__init__("hpo", context)
        ofactory = OntologyFactory()
        try:
            #sometimes the ontology world is down :(
            self.ont = ofactory.create('hp')
        except:
            logger.warn('Problem reaching sparql endpoint, falling back to obo')
            try:
                self.ont = ofactory.create('obo:hp')
            except:
                logger.error('Problem reaching obo, add local owl file')
                import sys
                sys.exit(1)
                #self.ont = ofactory.create('onto_cache/hpo.owl')
        #This seems to be required to make the ontology actually load:
        _ = self.ont.get_level(0)
        

    def hp_get_synonym(self,hp_identifier,curie_prefix):
        xref_ids = self.ont.xrefs(hp_identifier)
        doids = []
        for xref_id in xref_ids:
            if xref_id.startswith(curie_prefix):
                doids.append( xref_id )
        return doids

    def get_label(self,identifier):
        """Return the label for an identifier"""
        obj_ids = self.get_hp_id(identifier)
        return self.ont.label(obj_ids[0])

    def get_hp_id(self,obj_id):
        """Given an id, find the main key(s) that mondo uses for the id"""
        if self.ont.has_node(obj_id):
            obj_ids = [obj_id]
        else:
            obj_ids = self.ont.xrefs(obj_id, bidirectional=True)
        return obj_ids

    def has_ancestor(self,obj, terms):
        """Given an object and a term in MONDO, determine whether the term is an ancestor of the object.
        
        The object is a KNode representing a disease.
        Some complexity arises because the identifier for the node may not be the id of the concept in hp.
        the XRefs in hp are checked for the object if it is not intially found, but this may return more 
        than one entity if multiple hp entries map to the same.  

        Returns: boolean representing whether any hp objects derived from the subject have the term as an
                         ancestor.
                 The list of Mondo identifiers for the object, which have the term as an ancestor"""
        #TODO: The return signature is funky, fix it.
        obj_id = obj.identifier
        obj_ids = self.get_hp_id(obj_id)
        return_objects=[]
        for obj_id in obj_ids:
            ancestors = self.ont.ancestors(obj_id)
            for term in terms:
                if term in ancestors:
                    return_objects.append( obj_id )
        return len(return_objects) > 0, return_objects

    def substring_search(self,name):
        ciname = '(?i){}'.format(name)
        results = self.ont.search(ciname,synonyms=True,is_regex=True)

    def case_insensitive_search(self,name):
        ciname = '(?i)^{}$'.format(name)
        return self.ont.search(ciname,synonyms=True,is_regex=True)

    def search(self, name):
        #Exact match 
        results = self.case_insensitive_search(name)
        if len(results) == 0:
            if ',' in name:
                parts =name.split(',')
                parts.reverse()
                ps = [p.strip() for p in parts]
                newname = ' '.join(ps)
                results = self.case_insensitive_search(newname)
        return results



def test():
    h = HPO (ServiceContext.create_context ())
    #print( h.search('Arrhythmias, Cardiac') )
    print( h.search('Thyroid Neoplasms') )
    print( h.search('Neoplasm of the thyroid gland') )

if __name__ == '__main__':
    test()



import inspect
import json
import requests
import traceback
import unittest
from datetime import date
from datetime import datetime, timedelta
from bravado.client import SwaggerClient
from bravado.requests_client import RequestsClient
from string import Template
from uuid import getnode as get_mac
from greent.swagger import SwaggerEndpoint
from greent.service import Service
from greent.service import ServiceContext

class Clinical (Service):
    """ API for clinical data access. """

    def __init__(self, context):
        """ Initialize as a GreenT service. """
        super(Clinical, self).__init__("clinical", context)

    '''
    http://tweetsie.med.unc.edu/CLINICAL_EXPOSURE/age/{age}/
    http://tweetsie.med.unc.edu/CLINICAL_EXPOSURE/age/{age}/sex/{sex}/
    http://tweetsie.med.unc.edu/CLINICAL_EXPOSURE/age/{age}/sex/{sex}/race/{race}/
    http://tweetsie.med.unc.edu/CLINICAL_EXPOSURE/ageMin/{ageMin}/ageMax/{ageMax}/limit/{limit}
    '''
    def get_patients (self, age, sex=None, race=None, location=None):
        """ Call the clinical API the old fashioned way until the Swagger spec is fixed.
        """
        result = None
        url = self.url
        if sex != None:
            url = "{0}/{1}".format (url, "sex/${sex}")
            if race != None:
                url = "{0}/{1}".format (url, "race/${race}")
                if age != None:
                    url = "{0}/{1}".format (url, "age/${age}")
                    if location != None:
                        url = "{0}/{1}".format (url, "location/${location}/")
                    
        #url = Template ("http://tweetsie.med.unc.edu/CLINICAL_EXPOSURE/age/${age}/sex/${sex}/race/${race}/location/${location}/")
        query_string = Template (url).substitute (age = age, sex = sex, race = race, location = location)

        # The following test is about convenience for developers, not security. Security is enforced at lower levels with IP
        # address filtering. This code only allows a sample record to be returned for testing purposes when we're not on the
        # translator.ncats.io machine.
        translator_ncats_io_mac_address = 2773026512788
        mac_address = get_mac()
        print ("Our mac address: {0}".format (mac_address))
        live_api_enabled = mac_address == translator_ncats_io_mac_address
        if live_api_enabled:
            print ("query: {}".format (query_string))
            try:            
                r = requests.get (query_string)
                r.raise_for_status()
                result = r.json ()
            except requests.exceptions.RequestException as e:  # This is the correct syntax
                print (e)
        else:
            result = json.loads ("""[
         {
          "birth_date": "2006-08-02 00:00:00",
          "diag": {"ICD10:B08.4": {"2016-07-08 00:00:00": "OUTPATIENT"},
                   "ICD9:V19.2": {"2006-12-19 00:00:00": "OUTPATIENT"}},
          "geoCode": {"GEO:LAT": "35.22056", "GEO:LONG": "-80.69664"},
          "medList": {"MDCTN:10427": "2016-03-10 00:00:00",
                      "MDCTN:9502": "2017-01-20 00:00:00"},
          "patient_id": "32227752",
          "race": "white",
          "sex": "M"
         },
         {
          "birth_date": "2006-08-02 00:00:00",
          "diag": {"ICD10:B08.4": {"2016-07-08 00:00:00": "OUTPATIENT"},
                   "ICD9:V19.2": {"2006-12-19 00:00:00": "OUTPATIENT"}},
          "geoCode": {"GEO:LAT": "35.22056", "GEO:LONG": "-80.69664"},
          "medList": {"MDCTN:10427": "2016-03-10 00:00:00",
                      "MDCTN:9502": "2017-01-20 00:00:00"},
          "patient_id": "32227753",
          "race": "white",
          "sex": "M"
         }
        ]""")
        return result

class TestClinical(unittest.TestCase):

    pass

    '''
    exposures = Exposures ()

    def test_get_coordinates (self):
        print ("Test get coordinates")
        coordinates = self.exposures.get_coordinates (
            exposure_type='pm25',
            latitude='35.9131996',
            longitude='-79.0558445',
            radius='10')
        for coordinate in coordinates:
            print ("  --coord: lat: {0} lon: {1}".format (coordinate.latitude, coordinate.longitude))

    def test_get_scores (self):
        print ("Test get scores")
        scores = self.exposures.get_scores (
            exposure_type='pm25',
            start_date=date(2010, 1, 1),
            end_date=date(2010, 1, 7),
            exposure_point='35.9131996,-79.0558445')
        for exposure in scores:
            print ("  --scores: start: {0} end: {1} type: {2} lat: {3} lon: {4} units: {5} value: {6}".format (
                exposure.start_time, exposure.end_time, exposure.exposure_type,
                exposure.latitude, exposure.longitude, exposure.units, exposure.value))

    def test_get_values (self):
        print ("Test get values")
        values = self.exposures.get_values (
            exposure_type='pm25',
            start_date=date(2010, 1, 1),
            end_date=date(2010, 1, 7),
            exposure_point='35.9131996,-79.0558445')
        for exposure in values:
            print ("  --values: start: {0} end: {1} type: {2} lat: {3} lon: {4} units: {5} value: {6}".format (
                exposure.start_time, exposure.end_time, exposure.exposure_type,
                exposure.latitude, exposure.longitude, exposure.units, exposure.value))
    '''
    
if __name__ == '__main__':
    unittest.main()



import json
import requests
from greent.service import Service
from greent.graph_components import KNode, KEdge
from greent import node_types

class OXO(Service):

    """ Generic id translation service. Essentially a highly generic synonym finder. """
    def __init__(self, context): #url="https://www.ebi.ac.uk/spot/oxo/api/search?size=500"):
        super(OXO, self).__init__("oxo", context)
        self.build_valid_curie_prefixes()

    def build_valid_curie_prefixes(self):
        """Query for the current valid list of input curies"""
        #size defaults to 40...
        url = "https://www.ebi.ac.uk/spot/oxo/api/datasources?size=10000"
        response = requests.get (url).json ()
        self.curies = set()
        for ds in response['_embedded']['datasources']:
            self.curies.add(ds['prefix'])
            self.curies.update( ds['alternatePrefix'] )
        self.curies.add('MESH')

    def is_valid_curie_prefix(self, cp):
        return cp in self.curies

    def request (self, url, obj):
        return requests.post (self.url,
                              data=json.dumps (obj, indent=2),
                              headers={ "Content-Type" : "application/json" }).json ()

    def query (self, ids, distance=2):        
        return self.request (
            url = self.url,
            obj = {
                "ids"           : ids,
                "mappingTarget" : [],
                "distance"      : str(distance),
                "size"          : 10000
            })
    
    def get_synonyms( self, identifier, distance=2 ):
        """ Find all synonyms for a curie for a given distance . """
        result = []
        response = self.query (ids=[ identifier ], distance=distance)
        searchResults = response['_embedded']['searchResults']
        if len(searchResults) > 0 and searchResults[0]['queryId'] == identifier:
            others = searchResults[0]['mappingResponseList']
        return others

    def get_specific_synonym( self, identifier, prefix, distance=2 ):
        synonyms = self.get_synonyms( identifier, distance )
        return list( filter( lambda x: x['targetPrefix'] == prefix ,synonyms) )

    def get_specific_synonym_expanding(self, identifier, prefix):
        for i in range(1,4):
            synonyms = self.get_specific_synonym( identifier, prefix, distance=i )
            if len(synonyms) > 0:
                return synonyms
        return []

    def mesh_to_other (self, mesh_id):
        """ Find connections from a mesh id to other vocabulary domains. """
        return get_synonyms( mesh_id )
    
    def compile_results(self, fname, ntype, searchResults):
        result = []
        for other in searchResults:
            result.append( ( KEdge('oxo',fname, is_synonym=True),
                           KNode(identifier=other['curie'], node_type=ntype  )) )
        return result

    def efo_to_doid(self, efo_node):
        searchResults = self.get_specific_synonym( efo_node.identifier, 'DOID' )
        return self.compile_results('efo_to_doid',node_types.DISEASE, searchResults)

    def efo_to_umls(self, efo_node):
        searchResults = self.get_specific_synonym( efo_node.identifier, 'UMLS' )
        return self.compile_results('efo_to_umls',node_types.DISEASE, searchResults)

    def umls_to_doid(self, umls_node):
        searchResults = self.get_specific_synonym( umls_node.identifier, 'DOID' )
        return self.compile_results('umls_to_doid',node_types.DISEASE, searchResults)

    def ncit_to_hp(self, ncit_node):
        searchResults = self.get_specific_synonym( ncit_node.identifier, 'HP' )
        return self.compile_results('efo_to_umls',node_types.DISEASE, searchResults)

def test():
    from service import ServiceContext
    oxo = OXO(ServiceContext.create_context())
    r=oxo.query(['CL:85'])
    import json
    print (json.dumps(r, indent=4) )

def test2():
    from service import ServiceContext
    oxo = OXO(ServiceContext.create_context())
    r=oxo.efo_to_doid(KNode('EFO:0000764', node_types.DISEASE))
    print( r )

def test3():
    from service import ServiceContext
    oxo = OXO(ServiceContext.create_context())
    r=oxo.get_synonyms('EFO:0000764')
    import json
    print( json.dumps(r,indent=2) )

def test4():
    from service import ServiceContext
    oxo = OXO(ServiceContext.create_context())
    r=oxo.get_specific_synonym_expanding('DOID:9352', 'ICD9CM')
    import json
    print( json.dumps(r,indent=2) )


if __name__ == '__main__':
    test4()

'''

Reference: a conversation with OXO:

General:

Request URL:http://www.ebi.ac.uk/spot/oxo/api/search?size=500
Request Method:POST
Status Code:200 OK
Remote Address:193.62.193.80:80
Referrer Policy:no-referrer-when-downgrade

Response Headers view source

Cache-Control:no-cache, no-store, max-age=0, must-revalidate
Content-Type:application/json;charset=UTF-8
Date:Mon, 25 Sep 2017 14:24:54 GMT
Expires:0
Pragma:no-cache
Server:Apache-Coyote/1.1
Strict-Transport-Security:max-age=0
Transfer-Encoding:chunked
X-Application-Context:application
X-Content-Type-Options:nosniff
X-Frame-Options:DENY
X-XSS-Protection:1; mode=block


Request Headers view source

Accept:application/json, text/javascript, */*; q=0.01
Accept-Encoding:gzip, deflate
Accept-Language:en-US,en;q=0.8
Connection:keep-alive
Content-Length:58
Content-Type:application/json; charset=UTF-8
Cookie:_ga=GA1.3.1876641436.1506348116; _gid=GA1.3.425452358.1506348116; _gat=1
Host:www.ebi.ac.uk
Origin:http://www.ebi.ac.uk
Referer:http://www.ebi.ac.uk/spot/oxo/search
User-Agent:Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.101 Safari/537.36
X-Requested-With:XMLHttpRequest

Query String Parameters view source

view URL encoded
size:500


Request Payload view source

{ids: ["MESH:D001249"], mappingTarget: [], distance: "2"}







RESPONSE:


{
  "_embedded" : {
    "searchResults" : [ {
      "queryId" : "MESH:D001249",
      "querySource" : null,
      "curie" : "MeSH:D001249",
      "label" : "Asthma",
      "mappingResponseList" : [ {
        "curie" : "MedDRA:10003553",
        "label" : "Neutrophilic asthma",
        "sourcePrefixes" : [ "UMLS" ],
        "targetPrefix" : "MedDRA",
        "distance" : 2
      }, {
        "curie" : "HP:0002099",
        "label" : "Asthma",
        "sourcePrefixes" : [ "HP" ],
        "targetPrefix" : "HP",
        "distance" : 1
      }, {
        "curie" : "UMLS:C3714497",
        "label" : "Reactive airway disease",
        "sourcePrefixes" : [ "HP", "UMLS" ],
        "targetPrefix" : "UMLS",
        "distance" : 2
      }, {
        "curie" : "OMIM:608584",
        "label" : "ASTHMA-RELATED TRAITS, SUSCEPTIBILITY TO, 2",
        "sourcePrefixes" : [ "EFO" ],
        "targetPrefix" : "OMIM",
        "distance" : 2
      }, {
        "curie" : "OMIM:611960",
        "label" : "ASTHMA-RELATED TRAITS, SUSCEPTIBILITY TO, 7",
        "sourcePrefixes" : [ "EFO" ],
        "targetPrefix" : "OMIM",
        "distance" : 2
      }, {
        "curie" : "SNOMEDCT:278517007",
        "label" : "Asthmatic bronchitis",
        "sourcePrefixes" : [ "BAO", "DOID", "UMLS" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "OMIM:600807",
        "label" : "ASTHMA, SUSCEPTIBILITY TO",
        "sourcePrefixes" : [ "EFO", "BAO", "DOID" ],
        "targetPrefix" : "OMIM",
        "distance" : 2
      }, {
        "curie" : "SNOMEDCT:266365004",
        "label" : "",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "SNOMEDCT:187687003",
        "label" : "Asthma",
        "sourcePrefixes" : [ "BAO", "DOID", "UMLS" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "OMIM:611064",
        "label" : "ASTHMA-RELATED TRAITS, SUSCEPTIBILITY TO, 5",
        "sourcePrefixes" : [ "EFO" ],
        "targetPrefix" : "OMIM",
        "distance" : 2
      }, {
        "curie" : "EFO:0000270",
        "label" : "asthma",
        "sourcePrefixes" : [ "EFO" ],
        "targetPrefix" : "EFO",
        "distance" : 1
      }, {
        "curie" : "KEGG:05310",
        "label" : "",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "KEGG",
        "distance" : 2
      }, {
        "curie" : "SNOMEDCT:155574008",
        "label" : "Asthma",
        "sourcePrefixes" : [ "BAO", "DOID", "UMLS" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "ICD10CM:J45.909",
        "label" : "Unspecified asthma, uncomplicated",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "ICD10CM",
        "distance" : 2
      }, {
        "curie" : "SNOMEDCT:266398009",
        "label" : "Asthma: [NOS] or [attack]",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "ICD10CM:J45.90",
        "label" : "Late onset asthma",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "ICD10CM",
        "distance" : 2
      }, {
        "curie" : "NCIt:C28397",
        "label" : "Asthma",
        "sourcePrefixes" : [ "EFO", "BAO", "DOID", "UMLS" ],
        "targetPrefix" : "NCIt",
        "distance" : 2
      }, {
        "curie" : "ICD9CM:493.9",
        "label" : "",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "ICD9CM",
        "distance" : 2
      }, {
        "curie" : "DOID:2841",
        "label" : "asthma",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "DOID",
        "distance" : 1
      }, {
        "curie" : "SNOMEDCT:195967001",
        "label" : "Hyperreactive airway disease",
        "sourcePrefixes" : [ "EFO", "BAO", "DOID", "HP", "UMLS" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "SNOMEDCT:195979001",
        "label" : "",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "ICD9CM:493",
        "label" : "Asthma",
        "sourcePrefixes" : [ "EFO", "BAO", "DOID", "UMLS" ],
        "targetPrefix" : "ICD9CM",
        "distance" : 2
      }, {
        "curie" : "SNOMEDCT:21341004",
        "label" : "Asthmatic bronchitis",
        "sourcePrefixes" : [ "BAO", "DOID", "UMLS" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "SNOMEDCT:155579003",
        "label" : "Asthma: [NOS] or [attack]",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "ICD10CM:J45",
        "label" : "Asthma",
        "sourcePrefixes" : [ "BAO", "DOID", "UMLS" ],
        "targetPrefix" : "ICD10CM",
        "distance" : 2
      }, {
        "curie" : "UMLS:C0004096",
        "label" : "Asthma",
        "sourcePrefixes" : [ "UMLS" ],
        "targetPrefix" : "UMLS",
        "distance" : 1
      }, {
        "curie" : "SNOMEDCT:991000119106",
        "label" : "Reactive airway disease",
        "sourcePrefixes" : [ "HP" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "OMIM:MTHU003537",
        "label" : "Asthma",
        "sourcePrefixes" : [ "UMLS" ],
        "targetPrefix" : "OMIM",
        "distance" : 2
      }, {
        "curie" : "SNOMEDCT:195983001",
        "label" : "(Asthma: [exercise induced] or [allergic NEC] or [NOS]) or (allergic bronchitis NEC)",
        "sourcePrefixes" : [ "BAO", "DOID" ],
        "targetPrefix" : "SNOMEDCT",
        "distance" : 2
      }, {
        "curie" : "OMIM:607277",
        "label" : "ASTHMA-RELATED TRAITS, SUSCEPTIBILITY TO, 1",
        "sourcePrefixes" : [ "EFO" ],
        "targetPrefix" : "OMIM",
        "distance" : 2
      } ],
      "_links" : {
        "self" : {
          "href" : "http://www.ebi.ac.uk/spot/oxo/api/terms/MeSH:D001249"
        },
        "mappings" : {
          "href" : "http://www.ebi.ac.uk/spot/oxo/api/mappings?fromId=MeSH:D001249"
        }
      }
    } ]
  },
  "_links" : {
    "self" : {
      "href" : "http://www.ebi.ac.uk/spot/oxo/api/search"
    }
  },
  "page" : {
    "size" : 500,
    "totalElements" : 1,
    "totalPages" : 1,
    "number" : 0
  }
}
'''



from biothings_client import get_client
from BioThingsExplorer import pathViewer

t = pathViewer ()

mv = get_client("variant")
print (mv)

print (mv.getvariant("chr7:g.140453134T>C"))

mg = get_client("gene")
print (mg)

mg.getgene(1017, 'name,symbol,refseq')

md = get_client("drug")
print (md)

md.getdrug("ATBDZSAENDYQDW-UHFFFAOYSA-N", fields="pubchem")

mt = get_client("taxon")
print (mt)

print (mt.gettaxon(9606))


t.find_path ('drugname', 'wikipathways', excluded_nodes=['dbsnp', 'hgvs'])
             



import json
import logging
import operator
import os
import sys
import traceback
import unittest
import yaml
from enum import Enum
from greent.util import LoggingUtil
from greent.util import Resource
from greent.util import Text
from greent.util import DataStructure
from greent.service import Service
from greent.service import ServiceContext
from pprint import pformat, pprint
from neo4jrestclient.client import GraphDatabase

logger = LoggingUtil.init_logging (__file__, level=logging.DEBUG)

class TypeGraph(Service):
    """ A graph of 
           * nomenclature systems
           * conceptual domains in which they participate and
           * executable transitions translating from one nomenclature system to another
        Transitions specify the semantics by which operations convert between nomanclature systems.
        Each nomenclature system is referred to as a Type and recieves a label in the graph.
        Each concept is created as a node. Each type node with an associated concept 
           * Receives a label for the connected concept
           * Is the source of an is_a link connecting to the concept node.
        This enables queries between concept spaces to return alternative paths of operations
    """
    def __init__(self, service_context):
        """ Construct a type graph, registering labels for concepts and types. """
        super (TypeGraph, self).__init__("rosetta-graph", service_context)
        self.url = "{0}/db/data/".format (self.url)
        self.initialize_connection ()
        self.concepts = {}
        self.type_to_concept = {}
        self.concept_metadata = None
    def initialize_connection (self):
        logger.debug ("Creating type labels")
        self.db = GraphDatabase(self.url)
        self.types = self.db.labels.create("Type")
        self.concept_label = self.db.labels.create("Concept")
    def delete_all (self):
        """ Delete things in the graph. """
        try:
            with self.db.transaction (for_query=True, commit=True, using_globals=False) as transaction:
                transaction.query ("MATCH (n) DETACH DELETE n")
                results = transaction.query ("MATCH (n) RETURN n")
                print (results)
            self.initialize_connection ()
        except Exception as e:
            traceback.print_exc ()
    def set_concept_metadata (self, concept_metadata):
        """ Set the concept metadata. """
        logger.debug ("-- Initializing bio types.")
        self.concept_metadata = concept_metadata
        for concept, instances in self.concept_metadata.items ():
            self.concepts[concept] = self.db.labels.create (concept)
            for instance in instances:
                logger.debug ("Registering concept {} for instance {}".format (
                    concept, instance))
                self.type_to_concept [instance] = concept
    def find_or_create (self, name, iri=None):
        """ Find a type node, creating it if necessary. """
        n = self.types.get (name=name)
        if len(n) == 1:
            n = n[0]
        elif len(n) > 1:
            raise ValueError ("Unexpected non-unique node: {}".format (name))
        else:
            n = self.types.create (name=name, iri=iri)
            concept = self.type_to_concept.get (name)
            if concept:
                logger.debug ("   adding node {} to concept {}".format (name, concept))
                self.concepts[concept].add (n)
                concept_node = self._find_or_create_concept (concept)
                n.relationships.create ("is_a", concept_node)
        return n
    def add_edge (self, a, b, rel_name, predicate, op):
        """ Create a transition edge between two type nodes, storing the semantic predicate
        and transition operation. """
        a_node = self.find_or_create (a)
        b_node = self.find_or_create (b)
        a_rels = a_node.relationships.outgoing(rel_name, b_node)
        exists = False
        for rel in a_rels:
            if rel.properties.get ('op',None) == op:
                exists = True
        if not exists:
            enabled = predicate != "UNKNOWN"
            synonym = predicate == "SYNONYM"
            a_node.relationships.create (rel_name, b_node, predicate=predicate, op=op,
                                         enabled=enabled, synonym=synonym)
    def _find_or_create_concept (self, concept):
        """ Find or create a concept object which will be linked to member type object. """
        concept_node = self.concept_label.get (name=concept)
        if len(concept_node) == 1:
            logger.debug ("-- Loaded existing concept: {0}".format (concept))
            concept_node = concept_node[0]
        elif len(concept_node) > 1:
            raise ValueError ("Unexpected non-unique concept node: {}".format (concept))
        else:
            logger.debug ("-- Creating concept {0}".format (concept))
            concept_node = self.concept_label.create (name=concept)
        return concept_node
    def get_transitions (self, query):
        """ Execute a cypher query and walk the results to build a set of transitions to execute. """
        programs = []
        result = self.db.query (query, data_contents=True)
        if result.rows is None:
            return []
        for row_set in result.rows:
            program = []
            for row in row_set:
                #logger.debug (json.dumps (row, indent=2))
                node_type = None
                for col in row:
                    if isinstance (col, str):
                        node_type = col.split('>')[0] if '>' in col else col
                    elif isinstance(col, dict):
                        if 'name' in col:
                            #logger.debug ("  --result type: {0}".format (col))
                            node_type = col['name']
                        elif 'op' in col:
                            op = col['op']
                            predicate = col['predicate']
                            is_new = True
                            for level in program:
                                if level['node_type'] == node_type:
                                    level['ops'].append ({
                                        'link' : predicate,
                                        'op'   : op
                                    })
                                    is_new = False
                            if is_new:
                                program.append ({
                                    'node_type' : node_type,
                                    'ops'       : [
                                        {
                                            'link' : predicate,
                                            'op'   : op
                                        }
                                    ],
                                    'collector' : [] 
                                })
            programs.append (program)
        return programs



from collections import defaultdict
from csv import DictReader
import logging
import pronto
import requests
import os
from greent.graph_components import KNode,KEdge,elements_to_json
from greent import node_types
from greent.service import Service

class DiseaseOntology (Service):
    """ We use the Translator Disease Ontology service for other purposes but here, we fill a few gaps
    and keep our options open for digging into the DO further. Also, we map from DOID to Pharos disease ids. """

    def __init__(self, context): #obo_resource="http://purl.obolibrary.org/obo/doid.obo"):
        super(DiseaseOntology, self).__init__('diseaseontology', context)
        self.disease_ontology_data = 'doid.obo'
        self.initialized = False
        self.pharos_map = None
        self.pmap = None

    def load (self):
        """ Load the ontolgy. """
        if not os.path.exists (self.disease_ontology_data):
            url = self.url
            print ("Downloading disease ontology: {0}".format (url))
            r = requests.get(url, stream=True)
            with open(self.disease_ontology_data, 'wb') as f:
                for chunk in r.iter_content(chunk_size=1024): 
                    if chunk: # filter out keep-alive new chunks
                        f.write(chunk)

        self.ont = pronto.Ontology (self.disease_ontology_data)
        self.doid_to_mesh_map = defaultdict(lambda:[])
        for term in self.ont:
            xref = None
            if 'xref' in term.other:
                for xref in term.other['xref']:
                    if xref.startswith ("MESH:"):
                        self.doid_to_mesh_map[term.id.upper ()].append (xref)
        self.initialized = True

    def doid_to_mesh (self, doid):
        if not self.initialized:
            self.load ()
        return self.doid_to_mesh_map [doid]

    def graph_doid_to_mesh (self, doid):
        """ Convert a DOID to an equivalent MeSH term. """
        mesh_ids = self.doid_to_mesh (doid.identifier)
        return [ ( self.get_edge (predicate='synonym'),
                   KNode(identifier=i.replace ("MESH:", "MESH.DISEASE:"), node_type=node_types.DISEASE) ) for i in mesh_ids ]

    def doid_or_umls_to_pharos(self,doid):
        """ Convert a doid to a pharos id. Perhaps there's a public service that does this but in the
        mean time, we'll roll our own. """
        if not self.pmap:
            self.pmap = defaultdict(list)
            with open(os.path.join(os.path.dirname(__file__), 'pharos.id.all.txt'),'r') as inf:     #'pharos.id.txt','r') as inf:
                rows = DictReader(inf,dialect='excel-tab')
                for row in rows:
                    if row['DOID'] != '':
                        doidlist = row['DOID'].split(',')
                        for d in doidlist:
                            self.pmap[d.upper()].append(row['PharosID'])
        pharos_list = self.pmap[doid.identifier]
        if len(pharos_list) == 0:
            #logging.getLogger('application').warn('Unable to translate doid: %s into a Pharos ID' % doid)
            return []
        return list(map(lambda v : (
            KEdge('local','doid_to_pharos', is_synonym=True),
            KNode(identifier="PHAROS.DISEASE:{0}".format (v), node_type=node_types.DISEASE) ), pharos_list ))

def test():
    k = KNode("DOID:11476",node_types.DISEASE)
    from service import ServiceContext
    do = DiseaseOntology(ServiceContext.create_context())
    r=do.doid_or_umls_to_pharos(k)
    print (r)

    
if __name__ == "__main__":
    #do = DiseaseOntology ()
    #print (do.doid_to_mesh ("DOID:2841"))
    test()



import os
import yaml
from greent.util import Resource

class Config:
    def __init__(self, config):
        '''
        if not config.startswith (os.sep):
            config = os.path.join (os.path.dirname (__file__), config)
        '''
        config_path = Resource.get_resource_path (config)
        with open(config_path, 'r') as f:
            self.conf = yaml.safe_load (f)
    def get_service (self, service):
        return self.conf['translator']['services'][service]



import json
import logging
import os
import requests
import time
from collections import defaultdict
from greent.service import Service
from greent.service import ServiceContext
from greent.util import LoggingUtil
from greent.graph_components import KNode, KEdge
from greent.util import Text
from greent import node_types

logger = LoggingUtil.init_logging (__file__, level=logging.DEBUG)

class CTD (Service):
    """ """
    def __init__(self, context): 
        super(CTD, self).__init__("ctd", context)
        logger.debug ("Ensuring presence of CTD files: {0}".format (self.url))
        files = [
            'CTD_chem_gene_ixns.tsv','CTD_chemicals.tsv'
        ]
        for f in files:
            fname = os.path.join (os.path.dirname (__file__), f)
            if os.path.exists (fname):
                continue
            logger.debug ("  --downloading CTD component: {0}".format (f))
            gzname = fname+'.gz'
            url = "{0}/{1}.gz".format (self.url, f )
            r = requests.get (url, stream=True)
            with open (fname+'.gz', 'wb') as outf:
                for chunk in r.iter_content(chunk_size=1024): 
                    if chunk: # filter out keep-alive new chunks
                        outf.write (chunk)
            import gzip
            import shutil
            with gzip.open(fname+'.gz','rb') as f_in, open(fname,'wb') as f_out:
                shutil.copyfileobj(f_in,f_out)
        self.load_names()
        self.load_genes()

    def load_names(self):
        self.name_to_id = defaultdict(list)
        fname = os.path.join (os.path.dirname (__file__), 'CTD_chemicals.tsv') 
        with open(fname,'r') as inf:
            line = inf.readline()
            while line.startswith('#'):
                line = inf.readline()
            for line in inf:
                x = line.strip().split('\t')
                indexname = x[0]
                self.name_to_id[ indexname.lower() ].append(indexname)
                if len(x) > 7:
                    synonyms = x[7].split('|')
                    for synonym in synonyms:
                        self.name_to_id[ synonym.lower() ].append(indexname)

    def load_genes(self):
        self.drug_genes = defaultdict(list)
        fname = os.path.join (os.path.dirname (__file__), 'CTD_chem_gene_ixns.tsv') 
        with open(fname,'r') as inf:
            line = inf.readline()
            while line.startswith('#'):
                line = inf.readline()
            for line in inf:
                x = line.strip().split('\t')
                organismid = x[7]
                if organismid != '9606':
                    continue
                chemname = x[0]
                result = {}
                result['gene_id'] = 'NCBIGENE:{}'.format(x[4])
                if len(x) > 9:
                    result['actions'] = x[9].split('|')
                else:
                    result['actions'] = []
                if len(x) > 10:
                    result['publications'] = x[10].split('|')
                else:
                    result['publications'] = []
                self.drug_genes[chemname].append(result)

    def drugname_string_to_ctd_string(self,drugname):
        """This is exposed so that it can be used to look up names without the KNode structure"""
        identifiers = self.name_to_id[ drugname.lower() ]
        results = [ 'CTD:{}'.format(ident) for ident in identifiers ]
        return results

    def drugname_to_ctd(self, namenode):
        drugname = Text.un_curie(namenode.identifier)
        ctdids = self.drugname_string_to_ctd_string(drugname)
        for ctd in ctdids:
            label = drugname
            newnode = KNode( ctdid, node_types.DRUG, label=label)
            newedge = KEdge( 'CTD', 'drugname_to_ctd', {})
            results.append( (newedge, newnode ) )
        return results

    def drug_to_gene(self, subject):
        """ Get a gene from a ctd drug id. """
        ctdid = Text.un_curie (subject.identifier)
        actions = set()
        edge_nodes=[]
        for link in self.drug_genes[ctdid]:
            target_id = link['gene_id']
            edge_properties = { 'actions' : link['actions'],
                                'publications': link ['publications'] }
            actions.update( link['actions'] )
            edge = KEdge( 'ctd', 'drug_get_gene', {'properties': edge_properties } )
            node = KNode (target_id, node_types.GENE)
            edge_nodes.append( (edge, node) )
#        for action in actions:
#            print( 'Action: {}'.format(action) )
        return edge_nodes



def test_all_drugs():
    from greent.service import ServiceContext
    ctd = CTD(ServiceContext.create_context())
    with open('q2-drugandcondition-list.txt','r') as inf:
        h = inf.readline()
        uniq = set()
        for line in inf:
            x = line.split('\t')[0]
            uniq.add(x)
    n_no_ctd = 0
    n_no_gene = 0
    for name in uniq:
        input_node = KNode("DRUG_NAME:{}".format(name), node_types.DRUG_NAME)
        results = ctd.drugname_to_ctd(input_node)
        try:
            drug_node = results [0][1]
            ident = drug_node.identifier
        except:
            n_no_ctd += 1
            ident=''
            gene_nodes=[]
        if ident != '':
            gene_nodes = ctd.drug_to_gene( drug_node )
            if len(gene_nodes) == 0:
                n_no_gene += 1
        print('{}\t{}\t{}\t{}'.format(name, ident, len(results), len(gene_nodes) ))
    print( '{} drugs'.format(len(uniq)) )
    print( '{} without pubchem id'.format(n_no_ctd) )
    print( '{} without genes'.format(n_no_gene) )
    ngood = len(uniq) - n_no_ctd - n_no_gene
    print( '{} good ({})'.format( ngood, ngood/len(uniq) ) )

if __name__ == "__main__":
    test_all_drugs()



import argparse
import json
import logging
import networkx as nx
import networkx.algorithms as nxa
import operator
import os
import sys
import traceback
import unittest
import yaml
import requests
import requests_cache
from enum import Enum
from greent.util import LoggingUtil
from greent.util import Resource
from greent.util import Text
from greent.util import DataStructure
from greent.neo4j import Neo4JREST
from greent.service import Service
from greent.service import ServiceContext
from greent.graph import TypeGraph
from networkx.exception import NetworkXNoPath
from networkx.exception import NetworkXError
from pprint import pformat,pprint
from greent.graph_components import KNode,KEdge,elements_to_json
from networkx.readwrite import json_graph
from neo4jrestclient.client import GraphDatabase,Relationship,Node

logger = LoggingUtil.init_logging (__file__, level=logging.DEBUG)

class Rosetta:
    """ Rosetta's translates between semantic domains generically and automatically.
    Based on a configuration file, it builds a directed graph where types are nodes.
    Types can be IRIs or CURIEs. Edges are annotated with the names of operators used
    to transition between the connected types. The engine can then accept requests to 
    translate a term from one domain to another. It does this by collecting transitions
    from the graph and executing the list of transitions. """
    
    def __init__(self, greentConf="greent.conf",
                 config_file=os.path.join (os.path.dirname (__file__), "rosetta.yml"),
                 override={},
                 delete_type_graph=False,
                 init_db=False):

        """ The constructor loads the config file an prepares the type graph. If the delete_type_graph 
        flag is true, the graph is deleted entirely. If the init_db flag is true, the type_graph will
        be loaded from the config file. """
        """ Load the config file and set up a DiGraph representing the types we know 
        about and how to transition between them. """
        from greent.core import GreenT
        self.debug = False
        self.cache_path = 'rosetta_cache'

        logger.debug ("-- Initialize GreenT service core.")
        self.core = GreenT (config=greentConf, override=override)

        logger.debug ("-- Loading Rosetta graph schematic config: {0}".format (config_file))
        with open (config_file, 'r') as stream:
            self.config = yaml.load (stream)
            
        logger.debug ("-- Initializing vocabulary and curies.")
        self.curie = {}
        self.to_curie_map = {}
        self.vocab = self.config["@vocab"]
        for k in self.vocab:
            self.to_curie_map[self.vocab[k]] = k

        logger.debug ("-- Initializing Rosetta type graph")
        self.concepts = self.config["@concepts"]
        self.type_graph = TypeGraph (self.core.service_context)

        logger.debug ("-- Extending curie map with uber_context.")
        uber = Resource.get_resource_obj (os.path.join ("jsonld", "uber_context.jsonld"))
        context = uber['@context']
        self.terminate (context)
        for key, value in context.items ():
            self.curie[k] = value
            if isinstance (value, str):
                self.vocab[k] = value

        logger.debug ("-- Merge Identifiers.org vocabulary into Rosetta vocab.")
        identifiers_org = Resource.get_resource_obj ('identifiers.org.json')
        for module in identifiers_org:
            curie = module['prefix'].upper ()
            url = module['url']
            self.curie[curie] = url
            self.to_curie_map[url] = curie
            self.vocab[curie] = url

        if delete_type_graph:
            logger.debug ("--Deleting type graph")
            self.type_graph.delete_all ()
        
        if not init_db:
            return

        logger.debug ("--Initialize concept graph metadata and create type nodes.")
        self.type_graph.set_concept_metadata (self.concepts)
        for k, v in self.vocab.items ():
            if isinstance (v, str):
                self.type_graph.find_or_create (k, v)
        
        logger.debug ("-- Initializing Rosetta transition graph.")
        transitions = self.config["@transitions"]
        errors = 0
        for L in transitions:
            for R in transitions[L]:
                if not L in self.vocab:
                    errors += 1
                    self.log_debug("{0} not in vocab.".format (L))
                    continue
                if not R in self.vocab:
                    errors += 1
                    self.log_debug ("{0} not in vocab.".format (R))
                    continue
                assert L in self.vocab and R in self.vocab
                transition_dict = transitions[L][R]
                transition_obj = DataStructure.to_named_tuple ('TransitionTuple', transitions[L][R])
                if 'link' in transition_dict and 'op' in transition_dict:
                    self.type_graph.add_edge (L, R,
                                              rel_name=transition_obj.link.upper (),
                                              predicate=transition_obj.link.upper (),
                                              op=transition_obj.op)
        if errors > 0:
            logger.error ("** Encountered {0} errors. exiting.".format (errors))
            sys.exit (errors)
            
        logger.debug ("-- Connecting to translator registry to the type graph.")
        '''
        subscriptions = self.core.translator_registry.get_subscriptions ()
        for s in subscriptions:
            t_a_iri = s[0]
            t_b_iri = s[1]
            method = s[2]
            op = "translator_registry.{0}".format (method["op"])
            if not 'link' in method or method['link'] == None:
                link='unknown' #continue
            link = link.upper ()
            t_a = self.make_up_curie (self.unterminate (t_a_iri))
            t_b = self.make_up_curie (self.unterminate (t_b_iri))
            if not t_a:
                logger.debug ("Unable to find curie for {}".format (t_b))
            elif not t_b:
                logger.debug ("Unable to find curie for {}".format (t_b))
            else:
                self.type_graph.find_or_create (t_a, iri=t_a_iri)
                self.type_graph.find_or_create (t_b, iri=t_b_iri)
                if link and op:
                    self.type_graph.add_edge (t_a, t_b, rel_name=link, predicate=link, op=op)
        '''
        from transreg import MethodMetadata
        self.core.translator_registry.set_rosetta (self)
        subscriptions = self.core.translator_registry.get_subscriptions ()
        for sub in subscriptions:
            in_curie = self.to_curie (self.unterminate (sub.in_type))
            out_curie = self.to_curie (self.unterminate (sub.out_type))
            op = "translator_registry.{0}".format (sub.op)
            link = sub.predicate if sub.predicate else "unknown"
            link = link.upper ()
            if not in_curie:
                logger.debug ("Unable to find curie for {}".format (sub.in_type))
            elif not out_curie:
                logger.debug ("Unable to find curie for {}".format (sub.out_type))
            else:
                if link and op:
                    print ("--------------> {} {}".format (in_curie, out_curie))
                    self.type_graph.add_edge (in_curie, out_curie, rel_name=link, predicate=link, op=op)
        
    def terminate (self, d):
        for k, v in d.items ():
            if isinstance(v, str) and not v.endswith ("/"):
                d[k] = "{0}/".format (v)

    def unterminate (self, text):
        return text[:-1] if text.endswith ('/') else text

    def curie_to_iri (self, curie):
        return self.curie[curie] if curie in self.curie else None
    
    def guess_type (self, thing, source=None):
        """ Look for a CURIE we know. If that doesn't work, try one of our locally made up vocab words. """
        if thing and not source and ':' in thing:
            curie = thing.upper ().split (':')[0]
            if curie in self.curie:
                source = self.curie[curie]
        if source and not source.startswith ("http://"):
            source = self.vocab[source] if source in self.vocab else None
        return source

    def map_concept_types (self, thing, object_type=None):
        """ Expand high level concepts into concrete types our data sources understand. """

        # Try the CURIE approach.
        the_type = self.guess_type (thing.identifier) if thing and thing.identifier else None

        # If that didn't work, get candiddate types based on the (abstract) node type.
        if thing and not the_type:
            the_type = self.concepts.get (thing.node_type, None)
            if the_type:
                # Attempt to map them down to IRIs
                the_type = [ self.vocab.get(t,t) for t in the_type ]

        # Systematize this:
        # If the concept type is disease but the curie is NAME, we don't have a DOID.
        if isinstance(the_type,str):
            # If we've ended up with just one string, make it a list for conformity of return type
            the_type = [ the_type ]

        result = the_type if the_type else self.concepts.get (object_type, [ object_type ])

        curie = Text.get_curie (thing.identifier) if thing else None
        if curie:
            result = [ self.make_up_curie (curie) ] #[ self.vocab[curie] ]
            #result = [ self.vocab[curie] ]

        return result

    def to_curie (self, text):
        return self.to_curie_map.get (self.unterminate (text), None)
    
    def make_up_curie (self, text):
        """ If we got one, great. Yay for standards. If not, get creative. This is legitimate and
        important because we can't have useful automated reasoning without granular semantics. But
        folks make more specific sub domain names which is probably essential for semantics and
        therefore automation. Until we arrive at a better approach, lets accept this approach and make up
        a curie if the service author thought it was important to have one."""
        curie = self.to_curie (text)
        if not curie:
            pieces = text.split ('/')
            last = pieces[-1:][0]
            curie = last.upper ()
        return curie
    
    def get_ops (self, names):
        """ Dynamically locate python methods corresponding to names configured for semantic links. """
        return operator.attrgetter(names)(self.core) if isinstance(names,str) else [
            operator.attrgetter(n)(self.core) for n in names ]
    
    def log_debug (self, text, cycle=0, if_empty=False):
        if cycle < 3:
            if (text and len(text) > 0) or if_empty:
                logger.debug ("{}".format (text))
                
    def graph (self, next_nodes, query):
        """ Given a set of starting nodes and a query, execute the query to get a set of paths.
        Each path reflects a set of transitions from the starting tokens through the graph.
        Each path is then executed and the resulting links and nodes returned. """
        programs = self.type_graph.get_transitions (query)
        result = []
        for program in programs:
            result += self.graph_inner (next_nodes, program)
        return result
    
    def graph_inner (self, next_nodes, program):
        #print ("program: {}".format (json.dumps (program, indent=2)))
        if not program or len(program) == 0:
            return []
        primed = [ { 'collector' : next_nodes } ] + program
        linked_result = []
        for index, level in enumerate (program):
            logger.debug ("--Executing level: {0}".format (level))
            operators = level['ops']
            collector = level['collector']
            for edge_node in primed[index]['collector']:
                for operator in operators:
                    op = self.get_ops (operator['op'])
                    try:
                        results = None
                        log_text = "  -- {0}({1})".format (operator['op'], edge_node[1].identifier)
                        source_node = edge_node[1]
                        with requests_cache.enabled("rosetta_cache"):
                            results = op (source_node)
                        for r in results:
                            edge = r[0]
                            if isinstance(edge,KEdge):
                                edge.predicate = operator['link']
                                edge.source_node = source_node
                                edge.target_node = r[1]
                                linked_result.append (edge)
                        logger.debug ("{0} => {1}".format (log_text, Text.short (results)))
                        for r in results:
                            if index < len(program) - 1:
                                if not r[1].identifier.startswith (program[index+1]['node_type']):
                                    logger.debug (
                                        "Operator {0} wired to return type: {1} returned node with id: {2}".format (
                                            operator, program[index+1]['node_type'], r[1].identifier))
                        collector += results
                    except Exception as e:
                        traceback.print_exc()
                        logger.error ("Error invoking> {0}".format (log_text))        
        return linked_result
            
    def clinical_outcome_pathway (self, drug=None, disease=None):
        blackboard = []
        from greent import node_types
        if disease:
            blackboard += self.graph (
                [ ( None, KNode('NAME.DISEASE:{0}'.format (disease), node_types.NAME_DISEASE) ) ],
                query=\
                """MATCH (a{name:"NAME.DISEASE"}),(b:GeneticCondition), p = allShortestPaths((a)-[*]->(b)) 
                WHERE NONE (r IN relationships(p) WHERE type(r)='UNKNOWN') 
                RETURN p""")
            blackboard += self.graph (
                [ ( None, KNode('NAME.DISEASE:{0}'.format (disease), 'D') ) ],
                query=\
                """MATCH (a{name:"NAME.DISEASE"}),(b:Gene), p = allShortestPaths((a)-[*]->(b)) 
                WHERE NONE (r IN relationships(p) WHERE type(r)='UNKNOWN') 
                RETURN p""")
        if drug:
            blackboard += self.graph (
                [ ( None, KNode('NAME.DRUG:{0}'.format (drug), node_types.NAME_DRUG) ) ],
                query=\
                """MATCH (a{name:"NAME.DRUG"}),(b:Pathway), p = allShortestPaths((a)-[*]->(b)) 
                WHERE NONE (r IN relationships(p) WHERE type(r)='UNKNOWN') 
                RETURN p""")
        return blackboard
    
    @staticmethod
    def clinical_outcome_pathway_app (drug=None, disease=None, greent_conf='greent.conf'):
        return Rosetta(greentConf=greent_conf).clinical_outcome_pathway (drug=drug, disease=disease)

    @staticmethod
    def clinical_outcome_pathway_app_from_args (args, greent_conf='greent.conf'):        
        result = []
        if isinstance(args,list) and len(args) == 2 and \
           isinstance(args[0],str) and isinstance(args[1],str):
            result = (
                args,
                Rosetta.clinical_outcome_pathway_app (
                    drug=args[0],
                    disease=args[1],
                    greent_conf=greent_conf) )
        return result
    
if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Rosetta.')
    parser.add_argument('--delete-type-graph',
                        help='Delete the graph of types and semantic transitions between them.',
                        action="store_true", default=False)
    parser.add_argument('--initialize-type-graph',
                        help='Build the graph of types and semantic transitions between them.',
                        action="store_true", default=False)
    parser.add_argument('-d', '--disease', help='A disease to analyze.', default=None)
    parser.add_argument('-s', '--drug', help='A drug to analyze.', default=None)
    args = parser.parse_args()
    
    rosetta = Rosetta (init_db=args.initialize_type_graph,
                       delete_type_graph=args.delete_type_graph)
    blackboard = Rosetta.clinical_outcome_pathway_app (drug=args.drug,
                                                       disease=args.disease)
    print ("output: {}".format (blackboard))



import json
import unittest
from pprint import pprint
from greent.neo4j import Neo4JREST
from greent.util import Munge
from greent.util import Text
from greent.graph_components import KEdge, KNode
from greent import node_types 
from greent.service import ServiceContext

class HetIO(Neo4JREST):

    def __init__(self, context): #url="https://neo4j.het.io"):
        super (HetIO, self).__init__("hetio", context)

    def munge_gene (self, gene):
        return gene.split ("/")[-1:][0] if gene.startswith ("http://") else gene

    def gene_to_anatomy (self, gene):
        result = self.query (
            "MATCH (a:Anatomy)-[ar]-(g:Gene) WHERE g.name='{0}' RETURN a, ar, g LIMIT 200".format (Text.un_curie (gene.identifier)),
            labels=['Anatomy'],
            node_properties=['identifier'])
        print (result)
        return [ ( self.get_edge ({ 'res', r }, predicate='involved_in'), KNode(r['identifier'],  node_types.ANATOMY) ) for r in result ]
    
    def gene_to_cell (self, gene):
        result = self.query (
            "MATCH (g:Gene)-[r]-(c:CellularComponent) WHERE g.name='{0}' RETURN g, r, c LIMIT 200".format (Text.un_curie (gene.identifier)),
            labels=['CellularComponent'],
            node_properties=['identifier'])
        return [ ( self.get_edge ({ 'res' : r }, predicate='affects'), KNode(r['identifier'], node_types.CELLULAR_COMPONENT) ) for r in result ]

    def gene_to_disease (self, gene):
        if not Text.get_curie(gene.identifier) in [ 'HGNC', 'UNIPROT', 'PHAROS' ]:
            return []
        result = self.query (
            "MATCH (d:Disease)-[a1]-(g:Gene) WHERE g.name='{0}' RETURN a1,d".format (Text.un_curie(gene.identifier)),
            labels=['Disease'])
        #        result = self.nodes_and_edges (result)
        for r in result:
            print (r)
            print (result)
            print (type(result))
        #print ("-------------------> {}".format (json.dumps (result, indent=2)))
        return [ ( self.get_edge ({ 'res' : r }, predicate='affects'), KNode(r['identifier'], node_types.DISEASE) ) for r in result ]
    
    def disease_to_phenotype (self, disease):
        query = """MATCH (d:Disease{identifier:'%s'})-[r]-(s:Symptom) RETURN d,r,s""" % (disease.identifier)
        result = self.query (query, labels=['Symptom'], node_properties=None)
        edge_node = []
        for r in result:
            if r['source'] == 'MeSH':
                edge_node.append ( ( self.get_edge ({ 'res' : r }, predicate='affects'), KNode("MESH:{0}".format (r['identifier']), node_types.PHENOTYPE) ) )
        return edge_node
    
        #return [ ( self.get_edge ({ 'res' : r }, predicate='affects'), KNode("MESH:{0}".format (r['identifier']), 'PH') ) for r in result ]
        
class TestHetIO(unittest.TestCase):

    h = HetIO (ServiceContext.create_context ())
    
    def test_anatomy (self):
        pprint (self.h.gene_to_anatomy (KNode('HGNC:TP53', node_types.GENE)))

    def test_cell (self):
        pprint (self.h.gene_to_cell (KNode('HGNC:7121', node_types.GENE)))

if __name__ == '__main__':
    
    het = HetIO (ServiceContext.create_context ())
    print (het.disease_to_phenotype (KNode('DOID:2841',node_types.DISEASE)))
    '''
    with open('hgnc-entrez', 'r') as stream:
        for line in stream:
            h, e, u = line.split ('\t')
            het.gene_to_anatomy (KNode('SOMETHING:{}'.format (e), node_types.GENE))
    '''
    #unittest.main ()


#MATCH (g:Gene)-[r]-(c:CellularComponent) WHERE g.name='HGNC:3263' RETURN g, r, c LIMIT 200



from functools import singledispatch
from greent.node_types import node_types
from greent.util import Text

class KNode():
    """Used as the node object in KnowledgeGraph.
    
    Instances of this class can be passed to WorldGraph/greent as query subjects/objects."""
    def __init__(self,identifier,node_type,label=None):
        if identifier.startswith('http'):
            identifier = Text.obo_to_curie(identifier)
        self.identifier = identifier
        self.label = label
        if node_type not in node_types:
            raise ValueError( 'node_type {} unsupported.'.format( node_type ) )
        self.node_type = node_type
        self.properties = {}
        self.mesh_identifiers = []
        self.synonyms = set()
    def add_synonym(self,synonymous_node):
        """Merge anther KNode (representing a synonym) into this KNode."""
        self.synonyms.add(synonymous_node.identifier)
        for propkey in synonymous_node.properties:
            if propkey in self.properties:
                #TODO: this is messy
                if type(self.properties[propkey]) != type(synonymous_node.properties[propkey]):
                    raise Exception('Problem merging properties {}, {}'.format(type(self.properties[propkey]),type(synonymous_node.properties[propkey])))
                if isinstance(self.properties[propkey],list):
                    self.properties[propkey] += synonymous_node.properties[propkey]
                elif isinstance(self.properties[propkey],set):
                    self.properties[propkey].update( synonymous_node.properties[propkey] )
                else:
                    self.properties[ propkey ] = [ self.properties[propkey], synonymous_node.properties[propkey] ]
            else:
                self.properties[ propkey ] = synonymous_node.properties[propkey]
    def __repr__(self):
        #return "KNode(id={0},type={1})".format (self.identifier, self.node_type)
        return "N({0},t={1})".format (self.identifier, self.node_type)
    def __str__(self):
        return self.__repr__()
    #Is using identifier sufficient?  Probably need to be a bit smarter.
    def __hash__(self):
        """Class needs __hash__ in order to be used as a node in networkx"""
        return self.identifier.__hash__()
    def __eq__(x,y):
        return x.identifier == y.identifier
    def to_json(self):
        """Used to serialize a node to JSON."""
        j = { 'identifier': self.identifier, \
              'node_type' : self.node_type }
        if self.layer_number is not None:
            j['layer_number'] = self.layer_number
        for key in self.properties:
            j[key] = self.properties[key]
        return j
    def get_shortname(self):
        """Return a short user-readable string suitable for display in a list"""
        if self.label is not None:
            return '%s (%s)' % (self.label, self.identifier)
        return self.identifier
    def get_exportable(self):
        """Returns information to make a simpler node in networkx.  Helps with finicky graphml writer"""
        export_properties = { 'identifier'   : self.identifier, \
                              'node_type'    : self.node_type,  \
                              'layer_number' : self.layer_number }
        if self.label is not None:
            export_properties['label'] = self.label
        for key in self.properties:
            export_properties[key] = 'See JSON for details'
        return self.get_shortname(), export_properties


class KEdge():
    """Used as the edge object in KnowledgeGraph.

    Instances of this class should be returned from greenT"""
    def __init__(self, edge_source, edge_function, properties = None, is_synonym=False, is_support=False):
        self.edge_source = edge_source
        self.source_node = None
        self.target_node = None
        self.edge_function = edge_function
        if properties is not None:
            self.properties = properties
        else:
            self.properties = {}
        self.is_synonym = is_synonym
        self.is_support = is_support
    def __key(self):
        return (self.source_node, self.target_node, self.edge_source, self.edge_function)
    def __eq__(x,y):
        return x.__key() == y.__key()
    def __hash__(self):
        return hash(self.__key())
    def long_form (self):
        return "E(src={0},type={1},srcn={2},destn={3})".format (self.edge_source, self.edge_function,
                                                                self.source_node, self.target_node)
    def __repr__(self):
        #return "KEdge(edge_source={0},edge_type={1})".format (self.edge_source, self.edge_type)
        return self.long_form ()
#        return "E(src={0},type={1})".format (self.edge_source, self.edge_type)
    def __str__(self):
        return self.__repr__()
    def to_json(self):
        """Used to serialize a node to JSON."""
        j = { 'edge_source'     : self.edge_source, \
              'edge_function'   : self.edge_function, \
              'is_synonym'      : self.is_synonym }
        for key in self.properties:
            j[key] = self.properties[key]
        return j
    def get_exportable(self):
        """Returns information to make a simpler node in networkx.  Helps with finicky graphml writer"""
        export_properties = { 'edge_source'     : self.edge_source, \
              'edge_function'   : self.edge_function, \
              'is_synonym'      : self.is_synonym }
        for key in self.properties:
            export_properties[key] = 'See JSON for details'
        return export_properties

  


##
# We want to be able to serialize our knowledge graph to json.  That means being able to serialize KNode/KEdge.
# We could sublcass JSONEncoder (and still might), but for now, this set of functions allows the default
# encoder to find the functions that return serializable versions of KNode and KEdge
##

@singledispatch
def elements_to_json(x):
    """Used by default in dumping JSON. For use by json.dump; should not usually be called by users."""
    #The singledispatch decorator allows us to register serializers in our edge and node classes.
    return str(x)
 
@elements_to_json.register(KNode)
def node_to_json(knode):
    """Routes JSON serialization requests to KNode member function.  Not for external use."""
    return knode.to_json()

@elements_to_json.register(KEdge)
def node_to_json(kedge):
    """Routes JSON serialization requests to KEdge member function.  Not for external use."""
    return kedge.to_json()

# END JSON STUFF




from string import Template
import json
import os
import logging
from greent.service import Service
from greent.service import ServiceContext
from greent.triplestore import TripleStore
from greent.util import LoggingUtil
from greent.util import Text
from greent.graph_components import KEdge, KNode
from greent import node_types
from pprint import pprint
from cachier import cachier
import datetime

logger = LoggingUtil.init_logging (__file__)

class UberonGraphKS(Service):
    """A knowledge source created by 1) Combining cell ontology, uberon, and
    HPO, 2) Reasoning over the total graph to realize many implicit edges.
    Created by Jim Balhoff"""

    def __init__(self, context): #triplestore):
        super(UberonGraphKS, self).__init__("uberongraph", context)
        self.triplestore = TripleStore (self.url)

    def query_uberongraph (self, query):
        """ Execute and return the result of a SPARQL query. """
        return self.triplestore.execute_query (query)


    def cell_get_cellname (self, cell_identifier):
        """ Identify label for a cell type
        :param cell: CL identifier for cell type 
        """
        text = """
        prefix CL: <http://purl.obolibrary.org/obo/CL_>
        select distinct ?cellLabel
        from <http://reasoner.renci.org/nonredundant>
        from <http://example.org/uberon-hp-cl.ttl>
        where {
                  $cellID rdfs:label ?cellLabel .
              }
        """
        results = self.triplestore.query_template( 
            inputs = { 'cellID': cell_identifier }, \
            outputs = [ 'cellLabel' ], \
            template_text = text \
        )
        return results


    def get_anatomy_parts(self, anatomy_identifier):
        """Given an UBERON id, find other UBERONS that are parts of the query"""
        if anatomy_identifier.startswith('http'):
            anatomy_identifier = Text.obo_to_curie(anatomy_identifier)
        text="""
        prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        prefix UBERON: <http://purl.obolibrary.org/obo/UBERON_>
        prefix BFO: <http://purl.obolibrary.org/obo/BFO_>
        select distinct ?part ?partlabel
        from <http://reasoner.renci.org/nonredundant> 
        from <http://example.org/uberon-hp-cl.ttl>
        where {
                $anatomy_id BFO:0000051 ?part .
                ?part rdfs:subClassOf* UBERON:0001062 .
                ?part rdfs:label ?partlabel .
        }
        """
        results = self.triplestore.query_template(  
            inputs  = { 'anatomy_id': anatomy_identifier }, \
            outputs = [ 'part', 'partlabel' ], \
            template_text = text \
        )
        return results


    def cell_to_anatomy (self, cell_identifier):
        """ Identify anatomy terms related to cells.

        :param cell: CL identifier for cell type 
        """
        text = """
        prefix CL: <http://purl.obolibrary.org/obo/CL_>
        prefix BFO: <http://purl.obolibrary.org/obo/BFO_>
        select distinct ?anatomyID ?anatomyLabel
        from <http://reasoner.renci.org/nonredundant>
        from <http://example.org/uberon-hp-cl.ttl>
        where {
                  $cellID rdfs:subClassOf*/BFO:0000050 ?anatomyID .
                  ?anatomyID rdfs:label ?anatomyLabel .
              }
        """
        results = self.triplestore.query_template( 
            inputs = { 'cellID': cell_identifier }, \
            outputs = [ 'anatomyID', 'anatomyLabel' ], \
            template_text = text \
        )
        return results

    def phenotype_to_anatomy (self, hp_identifier):
        """ Identify anatomy terms related to cells.

        :param cell: HP identifier for phenotype
        """

        #The subclassof uberon:0001062 ensures that the result
        #is an anatomical entity.
        text = """
        prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        prefix UBERON: <http://purl.obolibrary.org/obo/UBERON_>
        prefix HP: <http://purl.obolibrary.org/obo/HP_>
        prefix part_of: <http://purl.obolibrary.org/obo/BFO_0000050>
        prefix has_part: <http://purl.obolibrary.org/obo/BFO_0000051>
        prefix depends_on: <http://purl.obolibrary.org/obo/RO_0002502>
        prefix phenotype_of: <http://purl.obolibrary.org/obo/UPHENO_0000001>
        select distinct ?anatomy_id ?anatomy_label ?input_label
        from <http://reasoner.renci.org/nonredundant>
        from <http://example.org/uberon-hp-cl.ttl>
        where {
                  ?anatomy_id rdfs:subClassOf* UBERON:0001062.
                  ?anatomy_id rdfs:label ?anatomy_label .
                  graph <http://reasoner.renci.org/nonredundant> {
                       ?phenotype phenotype_of: ?anatomy_id .
                  }
                  $HPID rdfs:subClassOf* ?phenotype .
                  $HPID rdfs:label ?input_label .
              }
        """
        results = self.triplestore.query_template( 
            inputs = { 'HPID': hp_identifier }, \
            outputs = [ 'anatomy_id', 'anatomy_label', 'input_label'],\
            template_text = text \
        )
        return results


    def get_anatomy_by_cell_graph (self, cell_node):
        anatomies = self.cell_to_anatomy (cell_node.identifier)
        results = []
        for r in anatomies:
            edge = KEdge ('uberongraph', 'cellToAnatomy')
            node = KNode (Text.obo_to_curie(r['anatomyID']), \
                   node_types.ANATOMY )
            node.label = r['anatomyLabel']
            results.append ( (edge, node) )
        return results
    
    def create_phenotype_anatomy_edge(self, node_id, node_label ):
        edge = KEdge ('uberongraph', 'phenotypeToAnatomy')
        node = KNode ( Text.obo_to_curie(node_id), \
               node_types.ANATOMY )
        node.label = node_label
        return edge,node

    def get_anatomy_by_phenotype_graph (self, phenotype_node):
        anatomies = self.phenotype_to_anatomy (phenotype_node.identifier)
        results = []
        for r in anatomies:
            edge, node = self.create_phenotype_anatomy_edge(r['anatomy_id'],r['anatomy_label'])
            if phenotype_node.label is None:
                phenotype_node.label = r['input_label']
            results.append ( (edge, node) )
            #These tend to be very high level terms.  Let's also get their parts to
            #be more inclusive.
            #TODO: there ought to be a more principled way to take care of this, but
            #it highlights the uneasy relationship between the high level world of
            #smartapi and the low-level sparql-vision.
            part_results = self.get_anatomy_parts( r['anatomy_id'] )
            for pr in part_results:
                pedge, pnode = self.create_phenotype_anatomy_edge(pr['part'],pr['partlabel'])
                results.append ( (pedge, pnode) )
        return results

def test_name():
    uk = UberonGraphKS(ServiceContext.create_context ())
    #Test cell->name
    cn ='CL:0000097'
    results = uk.cell_get_cellname( cn )
    print(results)
 

def test():
    uk = UberonGraphKS(ServiceContext.create_context ())
    #Test cell->anatomy
#    k = KNode('CL:0000097',node_types.CELL)
#    results = uk.get_anatomy_by_cell_graph( k )
#    print(results)
    #Test pheno->anatomy
    k = KNode('HP:0011675',node_types.PHENOTYPE)
    results = uk.get_anatomy_by_phenotype_graph( k )
    print(results)

def test_parts():
    uk = UberonGraphKS(ServiceContext.create_context ())
    print( uk.get_anatomy_parts('UBERON:0004535') )

if __name__ == '__main__':
    test()



DRUG='Substance'
GENE='Gene'
PATHWAY='Pathway'
PROCESS='BiologicalProcess'
CELL='Cell'
ANATOMY='Anatomy'
PHENOTYPE='Phenotype'
DISEASE='Disease'
GENETIC_CONDITION='GeneticCondition'
DRUG_NAME = 'NAME.DRUG'
DISEASE_NAME = 'NAME.DISEASE'

node_types = set([DRUG, GENE, PATHWAY, PROCESS, CELL, ANATOMY, PHENOTYPE, DISEASE, GENETIC_CONDITION, DRUG_NAME, DISEASE_NAME])

type_codes = { 'S': DRUG, 'G':GENE, 'P':PROCESS, 'C':CELL, 'A':ANATOMY, 'T':PHENOTYPE, 'D':DISEASE, 'X':GENETIC_CONDITION }



import json
import logging
import unittest
import requests
import sys
import traceback
import requests
from string import Template
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
from greent.util import LoggingUtil
from greent.endotype import Endotype

logger = LoggingUtil.init_logging (__file__)

class GraphQL (object):

    ''' GraphQL client for the Green Translator. '''
    def __init__(self, url="https://stars-app.renci.org/greent/graphql"):
        self.url = url

    def query (self, query):
        logger.debug ("query: %s", query)
        r = requests.post (self.url,
                           data = json.dumps (query).encode ('utf8'),
                           headers = { 'Content-Type': 'application/json' })
        r.raise_for_status ()
        return r.json ()

    # Exposures

    def get_exposure_scores (self, exposure_type, start_date, end_date, exposure_point):
        response = self.query ({
            "query" : """
            query getExposureScore($exposureType:String, $startDate:String, $endDate:String, $exposurePoint:String){
              exposureScore(exposureType  : $exposureType, 
    	        startDate     : $startDate, 
    	        endDate       : $endDate, 
    	        exposurePoint : $exposurePoint) {
                  value
                  latitude
                  longitude
                }
              }""",
            "variables" : {
                "exposureType"  : exposure_type,
                "startDate"     : start_date,
                "endDate"       : end_date,
                "exposurePoint" : exposure_point
            }
        })
        return response['data']['exposureScore']['scores']

    def get_exposure_values (self, exposure_type, start_date, end_date, exposure_point):
        response = self.query ({
            "query" : """
            query getExposureValue($exposureType:String, $startDate:String, $endDate:String, $exposurePoint:String){
              exposureValue(exposureType  : $exposureType, 
    	        startDate     : $startDate, 
    	        endDate       : $endDate , 
    	        exposurePoint : $exposurePoint) {
                  value
                  latitude
                  longitude
                }
              }""",
            "variables" : {
                "exposureType"  : exposure_type,
                "startDate"     : start_date,
                "endDate"       : end_date,
                "exposurePoint" : exposure_point
            }
        })
        return response['data']['exposureValue']

    # ChemBio API

    def get_exposure_conditions (self, chemicals):
        response = self.query ({
            "query" : """
            query getExposureConditions($chem : [String] ) {
              exposureConditions(chemicals: $chem) {
                chemical
                gene
                pathway
                pathName
                pathID
                human
              }
            }""",
            "variables" : {
                "chem" : chemicals
            }
        })
        return response ['data']['exposureConditions']
    
    def get_drugs_by_condition (self, conditions):
        response = self.query ({
            "query" : """
            query getDrugsByCondition($cond : [String] ) {
              drugsByCondition(conditions:$cond) {
                genericName
              }
            }""",
            "variables" : {
                "cond" : conditions
            }
        })
        drugs = response['data']['drugsByCondition']
        return [ r['genericName'] for r in drugs ]
 
    def get_genes_pathways_by_disease (self, diseases):
        response = self.query ({
            "query" : """
            query getGenesPathways($diseases : [String] ) {
              genePathsByDisease(diseases:$diseases) {
                uniprotGene
                keggPath
                pathName
                human
              }
            }""",
            "variables" : {
                "diseases" : diseases
            }
        })
        return response['data']['genePathsByDisease']

    # Clinical API

    def get_patients (self, age=None, sex=None, race=None, location=None):
        response = self.query ({
            "query" : """
            query queryPatients ($age : Int, $sex : String, $race : String) {
              patients (age: $age, sex: $sex, race: $race) {
                birthDate
                race
                sex
                patientId
                diagnoses {
                  diagnosis
                }
                geoCode {
                  latitude
                  longitude
                }
                prescriptions {
                  medication
                  date
                }
              }
            }""",
            "variables" : {
                "age"  : age,
                "sex"  : sex,
                "race" : race
            }
        })
        return response['data']['patients']

    def get_endotypes (self, query):
        response = self.query ({
            "query" : """
               query get_endotype ( $query : String) {
                 endotype (query:$query)
               }""",
            "variables" : {
                "query" : query
            }
        })
        return response['data']
    
    def translate (self, thing, domain_a, domain_b):
        query_text = Template ("""{ translate (thing:\"$thing\", domainA: \"$domain_a\", domainB: \"$domain_b\") { value } }""").\
           safe_substitute (thing=thing, domain_a=domain_a, domain_b=domain_b)
        result = self.query ({
            "query" : query_text
        })['data']['translate']
        return result
    
class TestGraphQLClient (unittest.TestCase):
 
    client = GraphQL ("http://localhost:5000/graphql")

    def test_get_exposure_scores (self):
        lat = "35.9131996"
        lon = "-79.0558445"
        result = self.client.get_exposure_scores (
            exposure_type = "pm25",
            start_date = "2010-01-01",
            end_date = "2010-01-07",
            exposure_point = ",".join ([ lat, lon ]))
        self.assertTrue (result['latitude'] == None)
        self.assertTrue (result['longitude'] == None)
        print ("Verified exposure response structure")

    def test_get_exposure_values (self):
        lat = "35.9131996"
        lon = "-79.0558445"
        result = self.client.get_exposure_values (
            exposure_type = "pm25",
            start_date = "2010-01-01",
            end_date = "2010-01-07",
            exposure_point = ",".join ([ lat, lon ]))
        self.assertTrue (result['latitude'] == None)
        self.assertTrue (result['longitude'] == None)
        print ("Verified exposure response structure")

    def test_get_exposure_conditions (self):
        result = self.client.get_exposure_conditions ([ "D052638" ])
        result = sorted (result, key=lambda v : v['gene'])
        self.assertTrue (result[0]['gene'] == 'http://chem2bio2rdf.org/uniprot/resource/gene/COL1A1')

    def test_get_drugs_by_condition (self):
        result = self.client.get_drugs_by_condition ([ "d001249" ])
        self.assertTrue ("Melphalan" in result)

    def test_get_genes_pathways_by_disease (self):
        result = self.client.get_genes_pathways_by_disease ([ "d001249" ])
        result = sorted (result, key=lambda v : v['uniprotGene'])
        self.assertTrue (result[0]['uniprotGene'] == 'http://chem2bio2rdf.org/uniprot/resource/gene/ABCB1')

    def test_clinical (self):
        pass #print (self.client.get_patients ())

    def test_endotype (self):
        greent = GraphQL ("http://localhost:5000/graphql")
        exposures = list(map(lambda exp : Endotype.create_exposure (**exp), [{
            "exposure_type": "pm25",
            "units"        : "",
            "value"        : 2
        }]))
        visits = list(map(lambda v : Endotype.create_visit(**v), [{
            "icd_codes"  : "ICD9:V12,ICD9:E002",
            "lat"        : "20",
            "lon"        : "20",
            "time"       : "2017-10-12 21:12:29",
            "visit_type" : "INPATIENT",
            "exposures"  : exposures
        }]))
        request = Endotype.create_request (dob= "2017-10-04", model_type="M0", race="1", sex="M", visits = visits)
        greent.get_endotypes (query = json.dumps (request))
        
if __name__ == '__main__':
    unittest.main()





from bravado.client import SwaggerClient
from greent.graph_components import KNode,KEdge,elements_to_json
from bravado.requests_client import RequestsClient
from greent.service import Service

class SwaggerEndpoint(Service):

    def __init__(self, name, context): #swagger_endpoint_url):
        super(SwaggerEndpoint, self).__init__(name, context)

        http_client = RequestsClient ()
        self.client = SwaggerClient.from_url(
            self.url, #swagger_endpoint_url,
            http_client=http_client,
            config={
                'use_models': False
            })

    def inspect (self):
        for name, obj in inspect.getmembers (self.client):
            print ("-- name: {0} obj: {1}".format (name, obj))
            for n, o in inspect.getmembers (obj):
                if (n.endswith ('_get') or n.endswith ('_post')):
                    print ("-- INVOKE: method-> {0} obj: {1}".format (n, o))

                    



import pronto
import requests
import os
from collections import defaultdict
from greent.graph_components import KEdge,KNode
from service import Service
from service import ServiceContext

class HumanPhenotypeOntology (Service):
    def __init__(self, context):
        super(HumanPhenotypeOntology, self).__init__('hpo', context)
        hpo_data = 'hpo.obo'
        if not os.path.exists (hpo_data):
            #url = "http://purl.obolibrary.org/obo/hp.obo"
            print ("Downloading human phenotype ontology: {0}".format (self.url))
            r = requests.get(self.url, stream=True)
            with open(hpo_data, 'wb') as f:
                for chunk in r.iter_content(chunk_size=1024): 
                    if chunk: # filter out keep-alive new chunks
                        f.write(chunk)
                    
        print ("Loading human phenotype ontology")
        self.ont = pronto.Ontology (hpo_data)
        self.mesh_to_pheno = defaultdict(lambda:[])
        for term in self.ont:
            xref = None
            if 'xref' in term.other:
                for xref in term.other['xref']:
                    if xref.startswith ("MSH:"):
                        logger.debug ("xref: {0} -> {1}".format (term.id, xref))
                        self.mesh_to_pheno[xref.upper()].append (term.id.upper()) #term.id.upper ()].append (xref)
        ''' print (ont.json) '''
        
    def mesh_to_phenotype (self, term):
        result = self.mesh_to_pheno.get (term.identifier.replace ('MESH:', 'MSH:'), []) \
                 if term.identifier.startswith ("MESH:") else []
        return [ ( self.get_edge({ 'mesh_id' : r }), KNode(r, node_types.DISEASE) ) for r in result ]
                   
    def get_term (self, term_id):
        return self.ont[term_id]
    

hpo = HumanPhenotypeOntology (ServiceContext.create_context ())
term = hpo.get_term ('HP:0002099')

def dump_term (t):
    print (t)
    #print (t['def'])
    for c in t.rchildren ():
        print ("  child: {}".format (c))
        #print ("  child: {}".format (c['def']))
        dump_term (c)
    for r in t.relations:
        print ("relation: {}".format (r))
        
print (hpo.mesh_to_phenotype (KNode("DOID:2841", node_types.DISEASE)))
print (hpo.mesh_to_phenotype (KNode("MESH:D001249", node_types.DISEASE)))



import asyncio
import concurrent.futures
import requests
import json
import logging
import sys
import traceback
import datetime
from cachier import cachier
from collections import defaultdict
from collections import namedtuple
from csv import DictReader
from greent.util import Munge
from greent.util import Text
from greent.service import Service
from greent.util import LoggingUtil
from greent.async import AsyncUtil
from greent.async import Operation
from greent.graph_components import KEdge, KNode
from greent import node_types
from simplejson.scanner import JSONDecodeError

logger = LoggingUtil.init_logging (__name__, logging.DEBUG)

class Pharos(Service):
    def __init__(self, context):
        super(Pharos,  self).__init__("pharos", context)
    def request (self, url):
        response = None
        try:
            response = requests.get (url).json ()
        except:
            traceback.print_exc ()
        return response
        
    def query (self, query):
        return self.request ("{0}/{1}".format (self.url, query))
    def target_to_id (self, target):
        target_map = self.query (query="targets/{0}".format (Munge.gene (target)))
        return target_map["id"] if "id" in target_map else None
    def target_to_disease (self, target_sym):
        target_id = self.target_to_id (Munge.gene (target_sym))
        links = self.query (query="targets({0})/links".format (target_id))
        result = []
        for k in links:
            if k['kind'] == "ix.idg.models.Disease":
                for p in k['properties']:
                    #print (p)
                    if p['label'] == 'IDG Disease':
                        result.append (p['term'])
        return result
    
    def disease_map (self, disease_id):
        return self.query (query="diseases({0})".format (disesase_id))

    def make_doid_id (self, obj):
        if not obj:
            return None
        result = obj
        if isinstance (obj, KNode):
            result = obj.identifier
        if isinstance (result, list):
            if len(result) == 1:
                result = result[0]
        if result:
            if isinstance(result, str):
                if result.startswith ('DOID:'):
                    result = result.replace ('DOID:', '')
        return result

    def translate(self, subject_node):
        """Convert a subject with a DOID or UMLS into a Pharos Disease ID"""
        #TODO: This relies on a pretty ridiculous caching of a map between pharos ids and doids.  
        #      As Pharos improves, this will not be required, but for the moment I don't know a better way.
        pmap = defaultdict(list)
        with open('pharos.id.txt','r') as inf:
            rows = DictReader(inf,dialect='excel-tab')
            for row in rows:
                if row['DOID'] != '':
                    doidlist = row['DOID'].split(',')
                    for d in doidlist:
                        pmap[d.upper()].append(row['PharosID'])
        doid = subject_node.identifier
        pharos_list = pmap[doid]
        if len(pharos_list) == 0:
            #logging.getLogger('application').warn('Unable to translate %s into Pharos ID' % doid)
            return None
        return pharos_list

    def target_to_hgnc(self, target_id):
        """Convert a pharos target id into an HGNC ID.
        The call does not return the actual name for the gene, so we do not provide it.
        There are numerous other synonyms that we could also cache, but I don't see much benefit here. """
        result = None
        try:
            r = requests.get('https://pharos.nih.gov/idg/api/v1/targets(%s)/synonyms' % target_id)
            result = r.json()
            for synonym in result:
                if synonym['label'] == 'HGNC':
                    result = synonym['term']
        except:
            pass
        return result

    def drugname_string_to_pharos_info(self,drugname):
        """Exposed for use in name lookups without KNodes"""
        r = requests.get('https://pharos.nih.gov/idg/api/v1/ligands/search?q={}'.format(drugname)).json()
        return [ ('PHAROS.DRUG:{}'.format(contents['id']),contents['name']) for contents in r['content'] ]

    def drugname_to_pharos(self, namenode):
        drugname  = Text.un_curie(namenode.identifier)
        pharosids = drugname_string_to_pharos_string(drugname)
        for pharosid, pharoslabel in pharosids:
            newnode = KNode( pharosid, node_types.DRUG, label=pharoslabel)
            newedge = KEdge( 'pharos', 'drugname_to_pharos', {} )
            results.append( (newedge, newnode ) )
        return results

    def disease_get_gene0(self, subject):
        """ Get a gene from a pharos disease id. """
        pharosids = self.translate (subject)
        print ("pharos ids: {}".format (pharosids))
        original_edge_nodes=[]
        for pharosid in pharosids:
            logger.debug ('pharos> https://pharos.nih.gov/idg/api/v1/diseases(%s)?view=full' % pharosid)
            r = requests.get('https://pharos.nih.gov/idg/api/v1/diseases(%s)?view=full' % pharosid)
            result = r.json()
            for link in result['links']:
                if link['kind'] != 'ix.idg.models.Target':
                    logger.info('Pharos disease returning new kind: %s' % link['kind'])
                else:
                    pharos_target_id = int(link['refid'])
                    pharos_edge = KEdge( 'pharos', 'disease_get_gene', {'properties': link['properties']} )
                    original_edge_nodes.append( (pharos_edge, pharos_target_id) )

#    @cachier(stale_after=datetime.timedelta(days=8))
#    def get_request (self, url):
#        return requests.get (url.json ())

#    @cachier(stale_after=datetime.timedelta(days=8))
    def drug_get_gene(self, subject):
        """ Get a gene from a pharos disease id. """
        pharosid = Text.un_curie (subject.identifier)
        original_edge_nodes=[]
        r = requests.get('https://pharos.nih.gov/idg/api/v1/ligands(%s)?view=full' % pharosid)
        result = r.json()
        resolved_edge_nodes = []
        actions = set() #for testing
        for link in result['links']:
            if link['kind'] == 'ix.idg.models.Target':
                pharos_target_id = int(link['refid'])
                edge_properties = {}
                for prop in link['properties']:
                    if prop['label'] == 'Pharmalogical Action': #!
                        actions.add(prop['term'] ) 
                pharos_edge = KEdge( 'pharos', 'drug_get_gene', {'properties': link['properties']} )               
                #Pharos returns target ids in its own numbering system. Collect other names for it.
                hgnc = self.target_to_hgnc (pharos_target_id)
                if hgnc is not None:
                    hgnc_node = KNode (hgnc, node_types.GENE)
                    resolved_edge_nodes.append( (pharos_edge, hgnc_node) )
                else:
                    logging.getLogger('application').warn('Did not get HGNC for pharosID %d' % pharos_target_id)
#        for a in actions:
#            print ('Action: {}'.format(a) ) 
        return resolved_edge_nodes

#    @cachier(stale_after=datetime.timedelta(days=8))
    def disease_get_gene(self, subject):
        """ Get a gene from a pharos disease id. """
        pharosid = Text.un_curie (subject.identifier)
        original_edge_nodes=[]
        r = requests.get('https://pharos.nih.gov/idg/api/v1/diseases(%s)?view=full' % pharosid)
        result = r.json()
        resolved_edge_nodes = []
        for link in result['links']:
            if link['kind'] == 'ix.idg.models.Target':
                pharos_target_id = int(link['refid'])
                pharos_edge = KEdge( 'pharos', 'disease_get_gene', {'properties': link['properties']} )               
                #Pharos returns target ids in its own numbering system. Collect other names for it.
                hgnc = self.target_to_hgnc (pharos_target_id)
                if hgnc is not None:
                    hgnc_node = KNode (hgnc, node_types.GENE)
                    resolved_edge_nodes.append( (pharos_edge, hgnc_node) )
                else:
                    logging.getLogger('application').warn('Did not get HGNC for pharosID %d' % pharos_target_id)
        return resolved_edge_nodes

class AsyncPharos(Pharos):
    """ Prototype asynchronous requests. In general we plan to have asynchronous requests and
    caching to accelerate query responses. """
    def __init__(self, context):
        super (AsyncPharos, self).__init__(context)
        
    def disease_get_gene(self, subject):
        pharosids = subject.identifier
        original_edge_nodes=[]
        def process_pharos_response (r):
            try:
                result = r.json()
                for link in result['links']:
                    if link['kind'] != 'ix.idg.models.Target':
                        logger.info('Pharos disease returning new kind: %s' % link['kind'])
                    else:
                        pharos_target_id = int(link['refid'])
                        pharos_edge = KEdge( 'pharos', 'disease_get_gene', {'properties': link['properties']} )
                        original_edge_nodes.append( (pharos_edge, pharos_target_id) )
            except JSONDecodeError as e:
                pass #logger.error ("got exception %s", e)
        AsyncUtil.execute_parallel_requests (
            urls=[ "https://pharos.nih.gov/idg/api/v1/diseases(%s)?view=full" % p for p in pharosids ],
            response_processor=process_pharos_response)

        logger.debug ("        Getting hgnc ids for pharos id: {}".format (pharosids))
        resolved_edge_nodes = []
        HGNCRequest = namedtuple ('HGNCRequest', [ 'pharos_target_id', 'edge' ])
        index = 0
        def process_hgnc_request (request):
            nonlocal index
            index += 1
            url = "https://pharos.nih.gov/idg/api/v1/targets(%s)/synonyms" % request.pharos_target_id
            if index < 3:
                logger.debug ("      hgnc_url:  {0}".format (url))
            return (requests.get (url).json (), request.edge)
        def process_hgnc_response (response):
            result = response[0]
            edge = response[1]
            hgnc = None
            for synonym in result:
                if synonym['label'] == 'HGNC':
                    hgnc = synonym['term']
            if hgnc is not None:
                hgnc_node = KNode(hgnc, node_types.GENE)
                resolved_edge_nodes.append((edge,hgnc_node))
        AsyncUtil.execute_parallel_operations (
            operations=[ Operation(process_hgnc_request, HGNCRequest(pharos_target_id, edge)) for edge, pharos_target_id in original_edge_nodes ], #[:1],
            response_processor=process_hgnc_response)
        
        return resolved_edge_nodes

#Poking around on the website there are about 10800 ( a few less )
def build_disease_translation():
    """Write to disk a table mapping Pharos disease ID to DOID (and other?) so we can reverse lookup"""
    with open('pharos.id.all.txt','w') as pfile:
        pfile.write('PharosID\tDOID\n')
        for pharosid in range(1,10800):
            r = requests.get('https://pharos.nih.gov/idg/api/v1/diseases(%d)/synonyms' % pharosid).json()
            doids = []
            for synonym in r:
                #if synonym['label'] == 'DOID':
                doids.append(synonym['term'])
            if len(doids) > 1:
                import json
                exit()
            elif len(doids) == 0:
                doids.append('')
            pfile.write('%d\t%s\n' % (pharosid, doids[0]))

def test_disese_gene_for_output():
    """Call a function so that we can examine the output"""
    pharosid=455
    r = requests.get('https://pharos.nih.gov/idg/api/v1/diseases(%d)?view=full' % pharosid)
    result = r.json()
    import json
    with open('testpharos.txt','w') as outf:
        json.dump(result,outf,indent=4)

def test_hgnc_for_output():
    """Call a function so that we can examine the output"""
    pharosid=91
    r = requests.get('https://pharos.nih.gov/idg/api/v1/targets(%d)/synonyms' % pharosid)
    result = r.json()
    import json
    with open('testpharos.txt','w') as outf:
        json.dump(result,outf,indent=4)

def test_drugs():
    in_node = KNode( 'NAME.DRUG:ADAPALENE', node_types.DRUG_NAME, label='ADAPALENE' )
    from service import ServiceContext
    pharos = Pharos(ServiceContext.create_context())
    results = pharos.drugname_to_pharos(in_node)
    for e,n in results:
        gres = pharos.drug_get_gene(n)
        for ge, gn in gres:
            print(gn)

def test_all_drugs():
    from greent.service import ServiceContext
    pharos = Pharos(ServiceContext.create_context())
    with open('q2-drugandcondition-list.txt','r') as inf:
        h = inf.readline()
        uniq = set()
        for line in inf:
            x = line.split('\t')[0]
            uniq.add(x)
    n_no_pharos = 0
    n_no_hgnc = 0
    for name in uniq:
        input_node = KNode("DRUG_NAME:{}".format(name), node_types.DRUG_NAME)
        try:
            results =  pharos.drugname_to_pharos(input_node)
            drug_node = results [0][1]
            ident = drug_node.identifier
            hgnc_nodes = pharos.drug_get_gene( drug_node )
            if len(hgnc_nodes) == 0:
                n_no_hgnc += 1
        except:
            #print ('Not finding {}'.format(name))
            #exit()
            n_no_pharos += 1
            ident=''
            hgnc_nodes=[]
        print('{}\t{}\t{}\t{}'.format(name, ident, len(results), len(hgnc_nodes) ))
    print( '{} drugs'.format(len(uniq)) )
    print( '{} without pubchem id'.format(n_no_pharos) )
    print( '{} without genes'.format(n_no_hgnc) )
    ngood = len(uniq) - n_no_pharos - n_no_hgnc
    print( '{} good ({})'.format( ngood, ngood/len(uniq) ) )



if __name__ == '__main__':
    #test_all_drugs()
    build_disease_translation()



import json
import requests
from greent import node_types
from greent.graph_components import KNode, KEdge
from greent.service import Service

class HGNC(Service):

    """ Generic GENE id translation service. Essentially a highly generic synonym finder. """
    def __init__(self, context): 
        super(HGNC, self).__init__("hgnc", context)

    def  get_name(self, node):
        """Given a node for an hgnc, return the name for that id"""
        if node.node_type != node_types.GENE:
            raise ValueError('Node must be a gene')
        identifier_parts = node.identifier.split(':')
        if identifier_parts[0] == 'HGNC':
            query_string='hgnc_id'
        elif identifier_parts[0].upper() == 'NCBIGENE':
            query_string = 'entrez_id'
        else:
            raise ValueError('Node must represent an HGNC or NCBIGene id.')
        hgnc_id = identifier_parts[1]
        headers = {'Accept':'application/json'}
        r = requests.get('%s/%s/%s' % (self.url, query_string, hgnc_id), headers= headers).json()
        symbol = r['response']['docs'][0]['symbol']
        return symbol 

    #todo, it would probably be straightforward to autogenerate these and have common logic for them
    def ncbigene_to_uniprotkb(self, node):
        """Given a node representing an NCBIGene (or Entrez) identifier, retrieve the UniProtKB identifier"""
        if node.node_type != node_types.GENE:
            raise ValueError('Node must be a gene')
        identifier_parts = node.identifier.split(':')
        if not identifier_parts[0].upper() == 'NCBIGENE':
            raise ValueError('Node must represent an NCBIGENE identifier.')
        hgnc_id = identifier_parts[1]
        headers = {'Accept':'application/json'}
        r = requests.get('{0}/entrez_id/{1}'.format(self.url, hgnc_id), headers= headers).json()
        try:
            uniprots = r['response']['docs'][0]['uniprot_ids']
            return  [  ( KEdge( 'hgnc', 'ncbigene_to_uniprotkb', is_synonym=True ),\
                         KNode( identifier='UNIPROTKB:{}'.format(uniprot), node_type = node_types.GENE )) \
                         for uniprot in uniprots ]
        except (IndexError, KeyError):
            #No results back
            return []

    def hgnc_to_uniprotkb(self, node):
        """Given a node representing an HGNC retrieve the UniProtKB identifier"""
        if node.node_type != node_types.GENE:
            raise ValueError('Node must be a gene')
        identifier_parts = node.identifier.split(':')
        if not identifier_parts[0].upper() == 'HGNC':
            raise ValueError('Node must represent an HGNC identifier.')
        hgnc_id = identifier_parts[1]
        headers = {'Accept':'application/json'}
        r = requests.get('{0}/hgnc_id/{1}'.format(self.url, hgnc_id), headers= headers).json()
        try:
            uniprots = r['response']['docs'][0]['uniprot_ids']
            return  [  ( KEdge( 'hgnc', 'ncbigene_to_uniprotkb', is_synonym=True ),\
                         KNode( identifier='UNIPROTKB:{}'.format(uniprot), node_type = node_types.GENE )) \
                         for uniprot in uniprots ]
        except (IndexError,KeyError):
            #No results back
            return []

def test():
    from greent.service import ServiceContext 
    hgnc = HGNC( ServiceContext.create_context() )
    input_knode = KNode( 'NCBIGENE:3815' , node_type = node_types.GENE )
    print( hgnc.ncbigene_to_uniprotkb( input_knode ) )

if __name__ == '__main__':
    test()




import json
import os
import unittest
from collections import defaultdict
from greent.mesh import MeSH
from greent.neo4j import Neo4JREST
from greent.util import LoggingUtil
from pprint import pprint
from greent.graph_components import KNode, KEdge
from greent import node_types

logger = LoggingUtil.init_logging (__file__)

class Chemotext(Neo4JREST):
    def __init__(self, context): #url="http://chemotext.mml.unc.edu:7474"):
        super (Chemotext, self).__init__("chemotext", context)
        self.mesh = MeSH ()
        self.cache = os.path.join(os.path.dirname(__file__),'chemotext.words.txt')
        if not os.path.exists(self.cache):
            build_synonym_cache(self)
        self.load_synonym_cache()

    def load_synonym_cache( self ):
        self.term_map = {}
        with open(self.cache,'r') as infile:
            h = infile.readline()
            for line in infile:
                x = line.strip().split('\t')
                self.term_map[x[0]] = x[1]
        
    def term_to_term (self, A, of_type=None, limit=100):
        response = self.query (
            query="MATCH (d:Term)-[r1]-(a:Art)-[r2]-(t:Term) WHERE d.name='%s' RETURN d, r1, a, r2, t LIMIT %s" % (A, limit))
        response = self.filter_nodes (response, labels=['Term'], properties=['name', 'type'])

        # Use MeSH data - slow but richer
        if of_type != None and response != None:
            new_response = []
            groups = defaultdict (lambda:None)
            for r in response:
                groups[r['name']] = r                
            for thing in groups:
                #logger.debug (" --mesh-req: {0}".format (thing))
                broader = self.mesh.get_broader (thing)
                obj = groups[thing]
                for k, v in of_type.items ():
                    for b in broader:
                        if k in b and b[k] == v:
                            #print ("b k v {0} {1} {2}".format (b, k, v))
                            obj['category'] = b['name']
                            obj['id'] = b['obj']
                            new_response.append (obj)
            response = new_response
        return response
    
    def disease_name_to_drug_name (self, disease, limit=100):
        result = []
        response = self.query (
            query="MATCH (d:Term {type:'Disease', name: '%s' })-[r1]-(a:Art)-[r2]-(t:Term {isDrug:true}) RETURN d, r1, a, r2, t LIMIT %s" %
            (disease, limit))
        for r in response['results'][0]['data']:
            result.append (r['row'][4]['name'])
        return list(set(list(result)))

    def graph_disease_name_to_drug_name (self, disease, limit=100):
        result = []
        drug_names = self.disease_name_to_drug_name (disease, limit)
        for r in drug_names:
            result.append ( ( self.get_edge (props=r), KNode("DRUGBANK.NAME:{0}".format (r['name']), node_types.NAME_DRUG) ) )
        return result

    def get_chemotext_term(self, input_term):
        qterm = input_term.upper()
        if qterm not in self.term_map:
            return None
        return self.term_map[qterm]
    

def build_synonym_cache(ctext = None):
    if ctext is None:
        from greent.service import ServiceContext
        ctext = Chemotext(ServiceContext.create_context())
    response = ctext.query( query="MATCH (d:Term) RETURN d")
    with open(ctext.cache,'w') as outfile:
        outfile.write('QUERY\tKEY\n')
        res = response['results'][0]
        n = 0
        for datum in res['data']:
            rows = datum['row']
            for row in rows:
                n+=1
                rowtype = row['type']
                meshname = row['name']
                if 'synonyms' in row:
                    rowsyn  = row['synonyms']
                else:
                    rowsyn = []
                outfile.write('{}\t{}\n'.format(meshname.upper(), meshname))
                for syn in rowsyn:
                    outfile.write('{}\t{}\n'.format(syn.upper(), meshname) )


'''
class TestChemotext(unittest.TestCase):

    chemotext = Chemotext (GreenT())
    
    def test_disease_to_drug (self):
        pprint ("Disease name to drug name:")
        pprint (self.chemotext.disease_name_to_drug_name ("Asthma"))

    def test_term_to_term (self):
        pprint (self.chemotext.term_to_term ('Asthma')[:10])

    def test_term_to_term_of_type (self):
        pprint ("term to term of type")
        pprint (self.chemotext.term_to_term ('Asthma', of_type={'name': 'Respiratory Hypersensitivity' }))
'''
if __name__ == '__main__':
    pass
    #build_synonym_cache()
    #unittest.main ()




from greent.service import Service
from greent.service import ServiceContext
from ontobio.ontol_factory import OntologyFactory

CELLULAR_COMPONENT='GO:0005575'
MOLECULAR_FUNCTION='GO:0005674'
BIOLOGICAL_PROCESS='GO:0008150'

#TODO: move commonalities here and with mondo into somehwere
class GO(Service):
    
    """ A pragmatic class to query the gene  ontology. Until better sources emerge, we roll our own. """ 
    def __init__(self, context ):
        super(GO, self).__init__("go", context)
        ofactory = OntologyFactory()
        try:
            #sometimes the ontology world is down :(
            self.ont = ofactory.create('go')
        except:
            self.ont = ofactory.create('obo:go')
        #self.ont = ofactory.create('./mondo.owl')
        #This seems to be required to make the ontology actually load:
        _ = self.ont.get_level(0)
        
    def get_label(self,identifier):
        """Return the label for an identifier"""
        return self.ont.label(identifier)

    def has_ancestor(self,identifier, term):
        """Determine whether a term has a particular ancestor"""
        #TODO: The return signature is funky, fix it.
        ancestors = self.ont.ancestors(identifier)
        return term in ancestors

    def is_cellular_component(self,identifier):
        """Checks go to find whether the subject is a cellular component"""
        return self.has_ancestor(identifier, CELLULAR_COMPONENT)
    def is_biological_process(self,identifier):
        """Checks go to find whether the subject is a cellular component"""
        return self.has_ancestor(identifier, BIOLOGICAL_PROCESS)
    def is_molecular_function(self,identifier):
        """Checks go to find whether the subject is a cellular component"""
        return self.has_ancestor(identifier, MOLECULAR_FUNCTION)




from distutils.core import setup
#from pip.req import parse_requirements
def parse_requirements(filename):
    """ load requirements from a pip requirements file """
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]

install_reqs = parse_requirements("greent/requirements.txt") #, session="i")
requirements = [str(r) for r in install_reqs]
setup(
    name = 'greent',
    packages = [ 'greent' ], # this must be the same as the name above
    package_data={ 'greent' : [
        'greent.conf',
        'greent-stars.conf',
        'rosetta.yml',
        'identifiers.org.json',
        'requirements.txt',
        'jsonld/*',
        'query/*.sparql'
    ]},
    version = '0.8',
    description = 'Green Team BioMedical Data Translator',
    author = 'Steve Cox',
    author_email = 'scox@renci.org',
    install_requires = requirements,
    include_package_data=True,
    url = 'https://github.com/NCATS-Tangerine/greent.git',
    download_url = 'http://github.com/NCATS-Tangerine/greent/archive/0.1.tar.gz',
    keywords = [ 'biomedical', 'environmental', 'exposure', 'clinical' ],
    classifiers = [ ],
)



import gensim.models
import json
import logging
import os
import requests
import time
from greent.service import Service
from greent.service import ServiceContext
from greent.util import LoggingUtil
from pprint import pformat

logger = LoggingUtil.init_logging (__file__, level=logging.DEBUG)

class Chemotext2 (Service):
    """ RENCI used the word2vec machine learning algorithm to compute teh semantic similarity of terms in the public access
    subset of the PubMed Central full text journal article corpus. Among other things, word2vec mdoels let us interrogate
    the "distance" between two terms. For a full explanation of the significance of distance, see the literature on word 
    embedding models and the word2vec algorithm. For our purposes, it's a sophisticated view of cooccurrence. """
    def __init__(self, context): #url="https://www.ebi.ac.uk/spot/oxo/api/search?size=500"):
        super(Chemotext2, self).__init__("chemotext2", context)
        logger.debug ("Ensuring presence of word to vec pubmed central models: {0}".format (self.url))
        files = [
            "pmc-2016.w2v", "pmc-2016.w2v.syn0.npy", "pmc-2016.w2v.syn1neg.npy",
            "bigram-2016.w2v", "bigram-2016.w2v.syn0.npy", "bigram-2016.w2v.syn1neg.npy"
        ]
        for f in files:
            if os.path.exists (f):
                continue
            logger.debug ("  --downloading word embedding model component: {0}".format (f))
            url = "{0}/{1}".format (self.url, f)
            r = requests.get (url, stream=True)
            with open (f, 'wb') as f:
                for chunk in r.iter_content(chunk_size=1024): 
                    if chunk: # filter out keep-alive new chunks
                        f.write (chunk)
        logger.debug ("All files present. Loading model. This will take a while.")
        model_path = files [0]

        start = time.time ()
        self.model = gensim.models.Word2Vec.load (model_path)
        logger.debug ("  -- loaded w2v term model: {0} in {1} seconds.".format (
            model_path, time.time () - start ))

        start = time.time ()
#       self.bigram_model = gensim.models.Word2Vec.load (model_path)
        logger.debug ("  -- loaded w2v bigram model: {0} in {1} seconds.".format (
            model_path, time.time () - start ))
        #with open ("a", "w") as stream:            
        #    stream.write (pformat (self.bigram_model.vocab))

    def get_semantic_similarity (self, term_a, term_b):
        """ Find semantic similarity of these terms as represented by a word2vec model generated from the 
        public access subset of PubMed Central full text journal articles. """
        term_a = term_a.lower ()
        term_b = term_b.lower ()
        result = None
        model = None
        if term_a.count(' ') == 1:
            raise ValueError ("We don't have a word embedding model for {0} word phrases".format (term_a.count(' ') + 1))
            model = self.bigram_model
            term_a = term_a.replace (' ', '_')
            term_b = term_b.replace (' ', '_')
        elif term_a.count (' ') == 0:
            model = self.model
        else:
            raise ValueError ("We don't have a word embedding model for {0} word phrases".format (term_a.count(' ') + 1))
            
        return model.similarity (term_a, term_b) if term_a in model.vocab and term_b in model.vocab else -1.0
            
        #return self.model.similarity (term_a, term_b) if term_a in self.model.vocab and term_b in self.model.vocab else -1.0


if __name__ == "__main__":
    ct2 = Chemotext2 (ServiceContext.create_context ())
    print (ct2.get_semantic_similarity ("lung cancer", "p53"))
    print (ct2.get_semantic_similarity ("cell line", "disease"))
    print (ct2.get_semantic_similarity ("cellular component", "nucleus"))
    print (ct2.get_semantic_similarity ("cell cycle", "krebbs"))

    print (ct2.get_semantic_similarity ("MAPK2", "P53"))

    w = [ "albuterol", "imatinib", "aspirin", "atrovent", "decadron", "medrol", "rayos" , "abemaciclib", "abraxane"]
    pairs = zip (w, w[1:])
    for k, v in pairs:
        print ((" k %s -> v %s : sim: %s" % (k, v, ct2.get_semantic_similarity (k, v))))
    print (ct2.model.most_similar (positive=['aspirin' ]))
    print (ct2.model.most_similar (positive=['p53' ]))
    print (ct2.model.most_similar (positive=['kit' ]))
    print (ct2.model.most_similar (positive=['asthma' ]))

    print (ct2.get_semantic_similarity('ebola', 'niemann'))
    print (ct2.get_semantic_similarity('Ebola', 'niemann'))
    print (ct2.get_semantic_similarity('ebola', 'Niemann'))
    print (ct2.get_semantic_similarity('ebola', 'niemann-pick'))
    print (ct2.get_semantic_similarity('niemann', 'pick'))
    print ('')
    print (ct2.get_semantic_similarity('Ebola Virus','niemann'))
    print (ct2.get_semantic_similarity('Ebola','niemann-pick disease'))
    print (ct2.get_semantic_similarity('Ebola virus','niemann-pick disease'))
    print ('')
    #This is pretty clear that the bigram is tough to use:
    print (ct2.get_semantic_similarity('Usher','WHRN'))           #gives 0.3
    print (ct2.get_semantic_similarity('Usher syndrome','WHRN'))  #gives -1
 




import datetime
import json
import logging
import os
import pprint
import unittest
from collections import defaultdict
from greent.biolink import Biolink
from greent.chembio import ChemBioKS
from greent.chemotext import Chemotext
from greent.ctd import CTD
from greent.clinical import Clinical
from greent.cmaq import CMAQ
from greent.config import Config
from greent.disease_ont import DiseaseOntology
from greent.endotype import Endotype
from greent.go import GO
from greent.hetio import HetIO
from greent.hgnc import HGNC
from greent.hpo import HPO
from greent.oxo import OXO
from greent.mondo import Mondo
from greent.pharos import Pharos
from greent.quickgo import QuickGo
from greent.service import ServiceContext
from greent.tkba import TranslatorKnowledgeBeaconAggregator
from greent.translator import Translator
from greent.transreg import TranslatorRegistry
from greent.triplestore import TripleStore
from greent.util import LoggingUtil
from greent.uberongraph import UberonGraphKS
from pprint import pprint

logger = LoggingUtil.init_logging (__file__)

class GreenT:

    ''' The Green Translator API - a single Python interface aggregating access mechanisms for 
    all Green Translator services. '''

    def __init__(self, config=None, override={}):
        self.service_context = ServiceContext.create_context (config)
        service_context = self.service_context
        
        self.clinical = Clinical (service_context)
        #temporarly taken out because of http errors
        #self.exposures = CMAQ (service_context)
        self.endotype = Endotype (service_context)

        self.chembio = ChemBioKS (self.service_context)
        self.chemotext = Chemotext (self.service_context)
        self.disease_ontology = DiseaseOntology (self.service_context)
        self.pharos = Pharos (self.service_context)
        self.oxo = OXO (self.service_context)
        self.hpo = HPO (self.service_context)
        self.hetio = HetIO (self.service_context)
        self.biolink = Biolink (self.service_context)
        self.mondo = Mondo(self.service_context)
        self.go = GO(self.service_context)
        self.tkba = TranslatorKnowledgeBeaconAggregator (self.service_context)
        self.translator_registry = TranslatorRegistry (self.service_context)
        self.quickgo = QuickGo (self.service_context)
        self.translator = Translator (core=self)
        self.hgnc = HGNC(self.service_context)
        self.uberongraph = UberonGraphKS(self.service_context)
        self.ctd = CTD(self.service_context)

    # Exposure API
    def get_exposure_scores (self, exposure_type, start_date, end_date, exposure_point):
        return self.exposures.get_scores (
            exposure_type = exposure_type,
            start_date    = start_date,
            end_date      = end_date,
            lat_lon       = exposure_point)

    def get_exposure_values (self, exposure_type, start_date, end_date, exposure_point):
        return self.exposures.get_values (
            exposure_type  = exposure_type,
            start_date     = start_date,
            end_date       = end_date,
            lat_lon        = exposure_point)

    # ChemBio API
    def get_exposure_conditions_json (self, chemicals):
        return json.dumps (self.get_exposure_conditions (chemicals))

    def get_exposure_conditions (self, chemicals):
        return self.chembio.get_exposure_conditions (chemicals)

    def get_drugs_by_condition_json (self, conditions):
        return json.dumps (self.get_drugs_by_condition (conditions))

    def get_drugs_by_condition (self, conditions):
        return self.chembio.get_drugs_by_condition (conditions)

    def get_genes_pathways_by_disease_json (self, diseases):
        return json.dumps (self.get_genes_pathways_by_disease (diseases))

    def get_genes_pathways_by_disease (self, diseases):
        return self.chembio.get_genes_pathways_by_disease (diseases)

    def get_drug_gene_disease (self, disease_name, drug_name):
        return self.chembio.get_drug_gene_disease (disease_name, drug_name)
    
    # Clinical API
    def get_patients (self, age=None, sex=None, race=None, location=None):
        return self.clinical.get_patients (age, sex, race, location)

    def execute (self, request):
        return self.translator.translate_chain (request)
